import fnet.data
from fnet.data.fnetdataset import FnetDataset
from fnet.data.tifreader import TifReader
import numpy as np
import fnet.transforms as transforms
import pandas as pd
import sys
import pdb
import os
import pickle
from scipy import stats

# output the second largest channel
def getsecondlarge(input):
    assert len(input[0].shape) == 3
    sorted_input = np.sort(input, axis = 0)    
    return sorted_input[-2]    

# output the index of the sorted input (dim=4)
def getindex(input):
    assert len(input[0].shape) == 3
    argsorted_input = np.argsort(input, axis = 0)
    return argsorted_input

# output the exclusivity array of differences between predictions for each organelle with the highest predicted value and the predictions of corresponding organelle with the second highest predicted value
def get_value(input, input_argsort):
    num_channel = len(input)
    channel = np.zeros(num_channel)
    length = np.zeros(num_channel)
    for i in range(num_channel):
        where = np.where(input_argsort[-1] == i)
        length[i] = len(where[0])
        channel[i] = np.sum(input[i][where] - getsecondlarge(input)[where])
    return channel, length
    
# output the exclusivity matrix of differences between predictions for each organelle with the highest predicted value and the predictions of the other organelle
def get_value3(input, input_argsort):
    num_channel = len(input)
    channel = np.zeros((num_channel, num_channel))
    length = np.zeros((num_channel, num_channel))
    for i in range(num_channel):
        where = np.where(input_argsort[-1] == i)
        for j in range(num_channel):
            length[i][j] = len(where[0])
            channel[i][j] = np.sum(input[i][where] - input[j][where])
    return channel, length

class TiffDataset2(FnetDataset):
    """Dataset for Tif files."""
    def __init__(self, dataframe: pd.DataFrame = None, path_csv: str = None,
                    transform_source = [transforms.normalize],
                    transform_target = None ):
        if dataframe is not None:
            self.df = dataframe
        else:
            self.df = pd.read_csv(path_csv)
        assert all(i in self.df.columns for i in ['path_1', 'path_2','path_3','path_4','path_5','path_6','path_7','path_8','path_9','path_10','path_11','path_12','path_target','path_prediction'])

        self.transform_source = transform_source
        self.transform_target = transform_target

    def __getitem__(self, index):
        element = self.df.iloc[index, :]
        im_out = []
        im_out2 = [TifReader(element['path_1']).get_image()]
        im_out2.append(TifReader(element['path_2']).get_image())
        im_out2.append(TifReader(element['path_3']).get_image())
        im_out2.append(TifReader(element['path_4']).get_image())
        im_out2.append(TifReader(element['path_5']).get_image())
        im_out2.append(TifReader(element['path_6']).get_image())
        im_out2.append(TifReader(element['path_7']).get_image())
        im_out2.append(TifReader(element['path_8']).get_image())
        im_out2.append(TifReader(element['path_9']).get_image())
        im_out2.append(TifReader(element['path_10']).get_image())
        im_out2.append(TifReader(element['path_11']).get_image())
        im_out2.append(TifReader(element['path_12']).get_image())
        im_out3 = [TifReader(element['path_target']).get_image()]
        im_out3.append(TifReader(element['path_prediction']).get_image())
        if self.transform_source is not None:
            for t in self.transform_source:
                im_out2[0] = t(im_out2[0])
                im_out2[1] = t(im_out2[1])
                im_out2[2] = t(im_out2[2])
                im_out2[3] = t(im_out2[3])
                im_out2[4] = t(im_out2[4])
                im_out2[5] = t(im_out2[5])
                im_out2[6] = t(im_out2[6])
                im_out2[7] = t(im_out2[7])
                im_out2[8] = t(im_out2[8])
                im_out2[9] = t(im_out2[9])
                im_out2[10]= t(im_out2[10])
                im_out2[11]= t(im_out2[11])
        if self.transform_target is not None:
            for t in self.transform_target:
                im_out3[0] = t(im_out3[0])
                im_out3[1] = t(im_out3[1])
        mse = np.square(im_out3[1]-im_out3[0]).mean()
        im_out.append(np.max(im_out2, axis=0))
        im_out.append(getsecondlarge(im_out2))
        indexes = getindex(im_out2)
        eval_list, num_list = get_value(im_out2, indexes)
        eval_matrix, num_matrix = get_value3(im_out2, indexes)
        return im_out, eval_matrix, num_matrix, eval_list, num_list, mse

    def correlation(self,index):
        element = self.df.iloc[index, :]
        im_out = []
        im_out3 = [TifReader(element['path_target']).get_image()]
        im_out3.append(TifReader(element['path_prediction']).get_image())
        if self.transform_target is not None:
            for t in self.transform_target:
                im_out3[0] = t(im_out3[0])
                im_out3[1] = t(im_out3[1])
        correlation, p = stats.pearsonr(im_out3[0].flatten(), im_out3[1].flatten())
        return correlation

    def __len__(self):
        return len(self.df)

    def get_information(self, index):
        return self.df.iloc[index, :].to_dict()

def main_correlation():
    data_type = sys.argv[1]
    model = sys.argv[2]
    param_idx = sys.argv[3]
    home_dir = sys.argv[4]
if data_type == 'train':
        dataset = TiffDataset2(path_csv=home_dir + '/csv/' + model + '/evaluate/' + model + '_evaluate_all_train_' + param_idx + '.csv')
        num_data_channel = [60, 60, 60, 60, 60, 60, 38, 60, 60, 60, 60, 60]
    elif data_type == 'test':
        dataset = TiffDataset2(path_csv=home_dir + '/csv/' + model + '/evaluate/' + model + '_evaluate_all_test_' + param_idx + '.csv')
        num_data_channel = [20, 20, 20, 20, 20, 20, 13, 20, 20, 20, 20, 20]
    
    num_channel = 12
    corr_value = [[]] #The first measurement
        # test: num_data_channel=[1,1,1,1,1,1,1,1,1,1,1,1]
    dataset_index = []
    for c in range(len(num_data_channel)):
        dataset_index += [c] * num_data_channel[c]
    for i in range(len(dataset)):
        data = dataset.correlation(i)
        corr_value[dataset_index[i]].append(data)
        if i <= len(dataset)-2:
            if dataset_index[i] != dataset_index[i+1]:
                corr_value.append([])
    print(corr_value)
    with open("evaluation_greedy/corr_improved_" + data_type + ".pickle", 'wb') as f:
        pickle.dump(corr_value, f)


def main():
    data_type = sys.argv[1]
    model = sys.argv[2]
    param_idx = sys.argv[3]
    home_dir = sys.argv[4]
    if data_type == 'train':
        dataset = TiffDataset2(path_csv=home_dir + '/csv/' + model + '/evaluate/' + model + '_evaluate_all_train_' + param_idx + '.csv')
        num_data_channel = [60, 60, 60, 60, 60, 60, 38, 60, 60, 60, 60, 60]
    elif data_type == 'test':
        dataset = TiffDataset2(path_csv=home_dir + '/csv/' + model + '/evaluate/' + model + '_evaluate_all_test_' + param_idx + '.csv')
        num_data_channel = [20, 20, 20, 20, 20, 20, 13, 20, 20, 20, 20, 20] 
    num_channel = 12
    eval_value = [] #The first measurement
    eval_list = np.zeros(num_channel) #The second measurement
    eval_matrix = np.zeros((num_channel,num_channel)) #The third measurement
    num_list = np.zeros(num_channel)
    num_matrix = np.zeros((num_channel, num_channel))
    mse_list = np.zeros(num_channel)
    mse = 0
        # test: num_data_channel=[1,1,1,1,1,1,1,1,1,1,1,1]
    dataset_index = []
    for c in range(len(num_data_channel)):
         dataset_index += [c] * num_data_channel[c]
    for i in range(len(dataset)):
        print(i)
        data = dataset[i]
        eval_value.append(np.mean(np.abs(data[0][0] - data[0][1])))
        eval_matrix += data[1]
        num_matrix += data[2]
        eval_list += data[3]
        num_list += data[4]
        mse += data[5]
        mse_list[dataset_index[i]] += data[5]
    eval_value_final = np.mean(np.array(eval_value))
    eval_list_final = np.true_divide(eval_list, num_list)
    eval_matrix_final = np.true_divide(eval_matrix, num_matrix)
    mse_list_final = np.array(mse_list) / np.array(num_data_channel)
    mse_final = mse / len(dataset)
    print(eval_value_final/mse_final)
    print(eval_value_final)
    print('[',end='')
    for r in range(num_channel):
        print(eval_list_final[r],';',sep='',end=' ')
    print(']')
    print('[',end='')
    for i in range(num_channel):
        for j in range(num_channel):
            print(eval_matrix_final[i][j],',',sep='',end=' ')
        print(';',end=' ')
    print(']')
    print(mse_final)
    print('[',end='')
    for r in range(num_channel):
        print(mse_list_final[r],';',sep='',end=' ')
    print(']')
    dir_name = "evaluation_greedy"
    if not os.path.exists(dir_name + "/" + model):
        os.mkdir(dir_name + "/" + model)
    final_dir=dir_name + "/" + model + "/" + data_type
    if not os.path.exists(final_dir):
        os.mkdir(final_dir)
    np.savetxt(final_dir + "/eval_value_final_" + data_type + "_" + param_idx + ".csv", np.array([eval_value_final]), delimiter=',')
    np.savetxt(final_dir + "/eval_list_final_" + data_type + "_" + param_idx + ".csv", eval_list_final, delimiter=',')
    np.savetxt(final_dir + "/eval_matrix_final_" + data_type + "_" + param_idx + ".csv", eval_matrix_final, delimiter=',')
    np.savetxt(final_dir + "/mse_list_final_" + data_type + "_" + param_idx + ".csv", mse_list_final, delimiter=',')
    np.savetxt(final_dir + "/mse_final_" + data_type + "_" + param_idx + ".csv", np.array([mse_final]), delimiter=',')


if __name__=="__main__":
    main()
    main_correlation()
