function answer = train_EBImages()
% Ulani Qi (uhq@andrew.cmu.edu)
%
% Copyright (C) 2018 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DO NOT MODIFY THIS BLOCK
if ~isdeployed()
  current_path = which(mfilename);
  [current_path, filename, extension] = fileparts( current_path );
%   get_murphylab_image_collections( true );
  cd(current_path);
end

dimensionality = '2D';

% other options
% -------------
options.verbose = true;
options.debug = true;
options.train.flag = 'framework';

% generic model options
% ---------------------
options.model.name = 'train_EBImages';
options.model.id = num2str(now);
options.model.filename = 'model.xml' ;

% nuclear shape model options
% ---------------------------
options.nucleus.type = 'diffeomorphic';
options.nucleus.class = 'framework';
options.nucleus.id = num2str(now);
options.nucleus.name = 'lamp2';

% cell shape model options
% ------------------------
options.cell.type = 'diffeomorphic';
options.cell.class = 'framework';
options.cell.model = 'lamp2';
options.cell.id = num2str(now);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FEEL FREE TO MODIFY THE VARIABLES IN THIS BLOCK

% bash command to generate function handles below:
% for f in img_cell_*.tif; do for I in {1..4}; do echo '@() im2uint8(im2bw(imread('\[\'cell\' filesep \'$f\'\], $I\), graythresh\(imread\('['\'cell\' filesep \'$f\'\], $I\)\)\)\), ...; done; done | pbcopy
% for f in img_dna_*.tif; do for I in {1..4}; do echo '@() im2uint8(im2bw(imread('\[\'dna\' filesep \'$f\'\], $I\), graythresh\(imread\('['\'dna\' filesep \'$f\'\], $I\)\)\)\), ...; done; done | pbcopy

dna={@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_1.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_1.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_1.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_1.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_1.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_1.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_1.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_1.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_10.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_10.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_10.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_10.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_10.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_10.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_10.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_10.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_100.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_100.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_100.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_100.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_100.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_100.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_100.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_100.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_101.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_101.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_101.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_101.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_101.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_101.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_101.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_101.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_102.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_102.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_102.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_102.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_102.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_102.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_102.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_102.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_103.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_103.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_103.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_103.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_103.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_103.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_103.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_103.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_104.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_104.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_104.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_104.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_104.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_104.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_104.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_104.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_105.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_105.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_105.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_105.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_105.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_105.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_105.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_105.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_106.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_106.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_106.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_106.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_106.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_106.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_106.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_106.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_107.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_107.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_107.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_107.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_107.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_107.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_107.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_107.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_108.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_108.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_108.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_108.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_108.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_108.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_108.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_108.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_109.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_109.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_109.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_109.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_109.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_109.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_109.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_109.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_11.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_11.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_11.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_11.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_11.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_11.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_11.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_11.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_110.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_110.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_110.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_110.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_110.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_110.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_110.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_110.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_111.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_111.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_111.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_111.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_111.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_111.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_111.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_111.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_112.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_112.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_112.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_112.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_112.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_112.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_112.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_112.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_113.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_113.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_113.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_113.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_113.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_113.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_113.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_113.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_114.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_114.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_114.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_114.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_114.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_114.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_114.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_114.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_115.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_115.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_115.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_115.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_115.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_115.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_115.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_115.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_116.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_116.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_116.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_116.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_116.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_116.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_116.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_116.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_117.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_117.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_117.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_117.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_117.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_117.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_117.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_117.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_118.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_118.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_118.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_118.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_118.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_118.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_118.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_118.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_119.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_119.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_119.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_119.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_119.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_119.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_119.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_119.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_12.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_12.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_12.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_12.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_12.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_12.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_12.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_12.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_120.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_120.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_120.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_120.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_120.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_120.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_120.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_120.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_121.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_121.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_121.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_121.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_121.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_121.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_121.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_121.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_122.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_122.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_122.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_122.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_122.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_122.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_122.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_122.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_123.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_123.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_123.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_123.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_123.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_123.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_123.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_123.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_124.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_124.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_124.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_124.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_124.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_124.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_124.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_124.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_125.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_125.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_125.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_125.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_125.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_125.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_125.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_125.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_126.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_126.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_126.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_126.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_126.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_126.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_126.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_126.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_127.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_127.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_127.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_127.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_127.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_127.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_127.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_127.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_128.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_128.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_128.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_128.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_128.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_128.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_128.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_128.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_129.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_129.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_129.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_129.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_129.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_129.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_129.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_129.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_13.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_13.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_13.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_13.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_13.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_13.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_13.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_13.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_130.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_130.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_130.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_130.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_130.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_130.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_130.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_130.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_131.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_131.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_131.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_131.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_131.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_131.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_131.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_131.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_132.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_132.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_132.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_132.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_132.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_132.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_132.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_132.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_133.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_133.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_133.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_133.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_133.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_133.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_133.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_133.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_134.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_134.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_134.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_134.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_134.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_134.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_134.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_134.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_135.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_135.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_135.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_135.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_135.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_135.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_135.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_135.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_136.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_136.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_136.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_136.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_136.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_136.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_136.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_136.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_137.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_137.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_137.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_137.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_137.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_137.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_137.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_137.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_138.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_138.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_138.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_138.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_138.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_138.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_138.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_138.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_139.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_139.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_139.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_139.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_139.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_139.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_139.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_139.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_14.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_14.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_14.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_14.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_14.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_14.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_14.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_14.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_140.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_140.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_140.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_140.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_140.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_140.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_140.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_140.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_141.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_141.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_141.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_141.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_141.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_141.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_141.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_141.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_142.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_142.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_142.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_142.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_142.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_142.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_142.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_142.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_143.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_143.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_143.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_143.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_143.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_143.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_143.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_143.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_144.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_144.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_144.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_144.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_144.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_144.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_144.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_144.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_145.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_145.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_145.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_145.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_145.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_145.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_145.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_145.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_146.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_146.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_146.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_146.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_146.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_146.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_146.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_146.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_147.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_147.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_147.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_147.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_147.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_147.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_147.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_147.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_148.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_148.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_148.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_148.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_148.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_148.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_148.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_148.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_149.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_149.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_149.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_149.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_149.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_149.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_149.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_149.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_15.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_15.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_15.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_15.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_15.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_15.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_15.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_15.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_150.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_150.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_150.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_150.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_150.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_150.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_150.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_150.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_151.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_151.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_151.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_151.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_151.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_151.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_151.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_151.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_152.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_152.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_152.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_152.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_152.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_152.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_152.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_152.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_153.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_153.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_153.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_153.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_153.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_153.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_153.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_153.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_154.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_154.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_154.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_154.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_154.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_154.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_154.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_154.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_155.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_155.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_155.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_155.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_155.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_155.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_155.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_155.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_156.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_156.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_156.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_156.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_156.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_156.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_156.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_156.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_157.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_157.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_157.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_157.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_157.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_157.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_157.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_157.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_158.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_158.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_158.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_158.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_158.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_158.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_158.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_158.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_159.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_159.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_159.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_159.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_159.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_159.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_159.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_159.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_16.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_16.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_16.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_16.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_16.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_16.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_16.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_16.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_160.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_160.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_160.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_160.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_160.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_160.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_160.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_160.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_161.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_161.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_161.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_161.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_161.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_161.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_161.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_161.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_162.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_162.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_162.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_162.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_162.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_162.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_162.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_162.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_163.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_163.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_163.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_163.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_163.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_163.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_163.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_163.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_164.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_164.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_164.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_164.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_164.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_164.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_164.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_164.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_165.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_165.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_165.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_165.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_165.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_165.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_165.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_165.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_166.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_166.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_166.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_166.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_166.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_166.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_166.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_166.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_167.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_167.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_167.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_167.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_167.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_167.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_167.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_167.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_168.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_168.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_168.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_168.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_168.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_168.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_168.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_168.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_169.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_169.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_169.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_169.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_169.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_169.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_169.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_169.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_17.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_17.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_17.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_17.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_17.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_17.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_17.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_17.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_170.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_170.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_170.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_170.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_170.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_170.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_170.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_170.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_171.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_171.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_171.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_171.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_171.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_171.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_171.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_171.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_172.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_172.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_172.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_172.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_172.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_172.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_172.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_172.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_173.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_173.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_173.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_173.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_173.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_173.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_173.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_173.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_174.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_174.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_174.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_174.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_174.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_174.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_174.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_174.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_175.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_175.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_175.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_175.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_175.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_175.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_175.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_175.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_176.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_176.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_176.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_176.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_176.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_176.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_176.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_176.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_177.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_177.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_177.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_177.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_177.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_177.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_177.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_177.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_178.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_178.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_178.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_178.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_178.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_178.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_178.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_178.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_179.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_179.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_179.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_179.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_179.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_179.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_179.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_179.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_18.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_18.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_18.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_18.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_18.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_18.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_18.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_18.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_180.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_180.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_180.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_180.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_180.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_180.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_180.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_180.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_181.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_181.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_181.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_181.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_181.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_181.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_181.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_181.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_182.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_182.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_182.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_182.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_182.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_182.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_182.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_182.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_183.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_183.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_183.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_183.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_183.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_183.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_183.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_183.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_184.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_184.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_184.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_184.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_184.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_184.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_184.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_184.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_185.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_185.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_185.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_185.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_185.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_185.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_185.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_185.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_186.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_186.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_186.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_186.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_186.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_186.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_186.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_186.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_187.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_187.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_187.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_187.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_187.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_187.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_187.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_187.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_188.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_188.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_188.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_188.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_188.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_188.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_188.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_188.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_189.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_189.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_189.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_189.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_189.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_189.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_189.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_189.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_19.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_19.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_19.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_19.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_19.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_19.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_19.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_19.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_190.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_190.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_190.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_190.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_190.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_190.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_190.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_190.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_191.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_191.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_191.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_191.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_191.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_191.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_191.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_191.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_192.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_192.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_192.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_192.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_192.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_192.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_192.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_192.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_193.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_193.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_193.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_193.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_193.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_193.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_193.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_193.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_194.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_194.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_194.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_194.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_194.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_194.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_194.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_194.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_195.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_195.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_195.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_195.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_195.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_195.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_195.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_195.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_196.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_196.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_196.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_196.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_196.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_196.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_196.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_196.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_197.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_197.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_197.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_197.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_197.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_197.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_197.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_197.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_198.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_198.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_198.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_198.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_198.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_198.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_198.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_198.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_199.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_199.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_199.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_199.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_199.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_199.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_199.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_199.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_2.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_2.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_2.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_2.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_2.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_2.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_2.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_2.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_20.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_20.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_20.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_20.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_20.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_20.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_20.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_20.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_200.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_200.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_200.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_200.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_200.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_200.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_200.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_200.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_201.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_201.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_201.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_201.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_201.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_201.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_201.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_201.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_202.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_202.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_202.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_202.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_202.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_202.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_202.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_202.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_203.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_203.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_203.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_203.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_203.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_203.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_203.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_203.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_204.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_204.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_204.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_204.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_204.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_204.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_204.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_204.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_205.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_205.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_205.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_205.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_205.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_205.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_205.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_205.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_206.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_206.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_206.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_206.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_206.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_206.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_206.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_206.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_207.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_207.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_207.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_207.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_207.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_207.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_207.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_207.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_208.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_208.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_208.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_208.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_208.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_208.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_208.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_208.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_209.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_209.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_209.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_209.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_209.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_209.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_209.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_209.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_21.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_21.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_21.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_21.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_21.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_21.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_21.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_21.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_210.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_210.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_210.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_210.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_210.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_210.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_210.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_210.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_211.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_211.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_211.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_211.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_211.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_211.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_211.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_211.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_212.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_212.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_212.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_212.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_212.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_212.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_212.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_212.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_213.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_213.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_213.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_213.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_213.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_213.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_213.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_213.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_214.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_214.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_214.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_214.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_214.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_214.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_214.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_214.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_215.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_215.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_215.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_215.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_215.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_215.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_215.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_215.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_216.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_216.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_216.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_216.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_216.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_216.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_216.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_216.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_217.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_217.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_217.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_217.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_217.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_217.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_217.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_217.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_218.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_218.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_218.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_218.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_218.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_218.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_218.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_218.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_219.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_219.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_219.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_219.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_219.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_219.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_219.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_219.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_22.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_22.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_22.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_22.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_22.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_22.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_22.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_22.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_220.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_220.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_220.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_220.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_220.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_220.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_220.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_220.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_221.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_221.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_221.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_221.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_221.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_221.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_221.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_221.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_222.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_222.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_222.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_222.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_222.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_222.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_222.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_222.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_223.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_223.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_223.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_223.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_223.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_223.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_223.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_223.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_224.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_224.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_224.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_224.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_224.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_224.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_224.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_224.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_225.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_225.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_225.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_225.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_225.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_225.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_225.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_225.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_226.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_226.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_226.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_226.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_226.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_226.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_226.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_226.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_227.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_227.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_227.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_227.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_227.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_227.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_227.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_227.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_228.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_228.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_228.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_228.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_228.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_228.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_228.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_228.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_229.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_229.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_229.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_229.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_229.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_229.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_229.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_229.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_23.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_23.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_23.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_23.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_23.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_23.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_23.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_23.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_230.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_230.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_230.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_230.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_230.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_230.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_230.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_230.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_231.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_231.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_231.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_231.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_231.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_231.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_231.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_231.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_232.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_232.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_232.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_232.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_232.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_232.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_232.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_232.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_233.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_233.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_233.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_233.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_233.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_233.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_233.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_233.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_234.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_234.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_234.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_234.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_234.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_234.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_234.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_234.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_235.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_235.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_235.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_235.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_235.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_235.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_235.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_235.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_236.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_236.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_236.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_236.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_236.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_236.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_236.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_236.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_237.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_237.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_237.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_237.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_237.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_237.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_237.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_237.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_238.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_238.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_238.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_238.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_238.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_238.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_238.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_238.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_239.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_239.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_239.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_239.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_239.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_239.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_239.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_239.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_24.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_24.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_24.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_24.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_24.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_24.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_24.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_24.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_240.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_240.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_240.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_240.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_240.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_240.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_240.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_240.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_241.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_241.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_241.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_241.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_241.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_241.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_241.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_241.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_242.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_242.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_242.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_242.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_242.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_242.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_242.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_242.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_243.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_243.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_243.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_243.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_243.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_243.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_243.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_243.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_244.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_244.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_244.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_244.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_244.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_244.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_244.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_244.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_245.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_245.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_245.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_245.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_245.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_245.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_245.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_245.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_246.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_246.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_246.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_246.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_246.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_246.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_246.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_246.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_247.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_247.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_247.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_247.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_247.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_247.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_247.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_247.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_248.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_248.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_248.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_248.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_248.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_248.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_248.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_248.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_249.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_249.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_249.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_249.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_249.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_249.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_249.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_249.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_25.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_25.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_25.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_25.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_25.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_25.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_25.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_25.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_250.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_250.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_250.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_250.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_250.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_250.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_250.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_250.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_251.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_251.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_251.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_251.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_251.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_251.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_251.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_251.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_252.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_252.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_252.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_252.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_252.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_252.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_252.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_252.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_253.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_253.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_253.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_253.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_253.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_253.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_253.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_253.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_254.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_254.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_254.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_254.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_254.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_254.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_254.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_254.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_255.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_255.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_255.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_255.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_255.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_255.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_255.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_255.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_256.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_256.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_256.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_256.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_256.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_256.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_256.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_256.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_257.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_257.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_257.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_257.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_257.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_257.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_257.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_257.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_258.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_258.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_258.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_258.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_258.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_258.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_258.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_258.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_259.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_259.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_259.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_259.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_259.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_259.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_259.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_259.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_26.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_26.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_26.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_26.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_26.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_26.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_26.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_26.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_260.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_260.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_260.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_260.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_260.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_260.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_260.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_260.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_261.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_261.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_261.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_261.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_261.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_261.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_261.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_261.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_262.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_262.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_262.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_262.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_262.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_262.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_262.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_262.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_263.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_263.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_263.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_263.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_263.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_263.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_263.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_263.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_264.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_264.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_264.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_264.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_264.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_264.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_264.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_264.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_265.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_265.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_265.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_265.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_265.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_265.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_265.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_265.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_266.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_266.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_266.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_266.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_266.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_266.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_266.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_266.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_267.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_267.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_267.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_267.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_267.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_267.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_267.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_267.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_268.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_268.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_268.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_268.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_268.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_268.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_268.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_268.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_269.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_269.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_269.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_269.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_269.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_269.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_269.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_269.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_27.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_27.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_27.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_27.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_27.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_27.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_27.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_27.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_270.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_270.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_270.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_270.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_270.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_270.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_270.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_270.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_271.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_271.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_271.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_271.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_271.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_271.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_271.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_271.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_272.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_272.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_272.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_272.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_272.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_272.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_272.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_272.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_273.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_273.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_273.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_273.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_273.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_273.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_273.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_273.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_274.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_274.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_274.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_274.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_274.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_274.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_274.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_274.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_275.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_275.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_275.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_275.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_275.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_275.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_275.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_275.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_276.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_276.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_276.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_276.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_276.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_276.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_276.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_276.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_277.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_277.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_277.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_277.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_277.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_277.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_277.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_277.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_278.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_278.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_278.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_278.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_278.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_278.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_278.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_278.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_279.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_279.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_279.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_279.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_279.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_279.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_279.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_279.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_28.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_28.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_28.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_28.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_28.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_28.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_28.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_28.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_280.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_280.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_280.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_280.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_280.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_280.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_280.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_280.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_281.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_281.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_281.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_281.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_281.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_281.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_281.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_281.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_282.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_282.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_282.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_282.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_282.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_282.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_282.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_282.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_283.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_283.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_283.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_283.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_283.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_283.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_283.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_283.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_284.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_284.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_284.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_284.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_284.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_284.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_284.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_284.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_285.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_285.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_285.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_285.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_285.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_285.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_285.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_285.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_286.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_286.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_286.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_286.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_286.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_286.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_286.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_286.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_287.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_287.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_287.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_287.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_287.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_287.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_287.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_287.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_288.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_288.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_288.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_288.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_288.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_288.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_288.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_288.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_289.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_289.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_289.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_289.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_289.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_289.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_289.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_289.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_29.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_29.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_29.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_29.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_29.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_29.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_29.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_29.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_290.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_290.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_290.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_290.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_290.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_290.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_290.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_290.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_291.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_291.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_291.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_291.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_291.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_291.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_291.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_291.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_292.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_292.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_292.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_292.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_292.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_292.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_292.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_292.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_293.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_293.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_293.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_293.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_293.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_293.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_293.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_293.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_294.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_294.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_294.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_294.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_294.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_294.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_294.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_294.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_295.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_295.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_295.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_295.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_295.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_295.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_295.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_295.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_296.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_296.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_296.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_296.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_296.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_296.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_296.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_296.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_297.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_297.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_297.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_297.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_297.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_297.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_297.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_297.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_298.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_298.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_298.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_298.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_298.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_298.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_298.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_298.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_299.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_299.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_299.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_299.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_299.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_299.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_299.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_299.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_3.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_3.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_3.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_3.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_3.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_3.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_3.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_3.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_30.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_30.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_30.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_30.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_30.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_30.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_30.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_30.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_300.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_300.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_300.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_300.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_300.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_300.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_300.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_300.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_301.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_301.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_301.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_301.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_301.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_301.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_301.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_301.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_302.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_302.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_302.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_302.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_302.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_302.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_302.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_302.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_303.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_303.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_303.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_303.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_303.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_303.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_303.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_303.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_304.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_304.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_304.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_304.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_304.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_304.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_304.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_304.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_305.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_305.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_305.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_305.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_305.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_305.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_305.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_305.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_306.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_306.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_306.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_306.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_306.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_306.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_306.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_306.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_307.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_307.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_307.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_307.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_307.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_307.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_307.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_307.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_308.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_308.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_308.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_308.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_308.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_308.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_308.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_308.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_309.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_309.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_309.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_309.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_309.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_309.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_309.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_309.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_31.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_31.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_31.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_31.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_31.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_31.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_31.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_31.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_310.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_310.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_310.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_310.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_310.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_310.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_310.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_310.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_311.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_311.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_311.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_311.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_311.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_311.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_311.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_311.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_312.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_312.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_312.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_312.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_312.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_312.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_312.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_312.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_313.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_313.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_313.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_313.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_313.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_313.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_313.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_313.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_314.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_314.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_314.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_314.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_314.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_314.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_314.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_314.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_315.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_315.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_315.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_315.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_315.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_315.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_315.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_315.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_316.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_316.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_316.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_316.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_316.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_316.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_316.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_316.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_317.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_317.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_317.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_317.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_317.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_317.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_317.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_317.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_318.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_318.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_318.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_318.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_318.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_318.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_318.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_318.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_319.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_319.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_319.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_319.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_319.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_319.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_319.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_319.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_32.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_32.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_32.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_32.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_32.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_32.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_32.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_32.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_320.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_320.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_320.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_320.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_320.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_320.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_320.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_320.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_321.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_321.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_321.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_321.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_321.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_321.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_321.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_321.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_322.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_322.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_322.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_322.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_322.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_322.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_322.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_322.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_323.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_323.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_323.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_323.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_323.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_323.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_323.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_323.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_324.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_324.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_324.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_324.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_324.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_324.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_324.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_324.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_325.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_325.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_325.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_325.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_325.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_325.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_325.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_325.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_326.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_326.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_326.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_326.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_326.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_326.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_326.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_326.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_327.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_327.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_327.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_327.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_327.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_327.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_327.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_327.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_328.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_328.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_328.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_328.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_328.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_328.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_328.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_328.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_329.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_329.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_329.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_329.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_329.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_329.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_329.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_329.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_33.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_33.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_33.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_33.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_33.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_33.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_33.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_33.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_330.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_330.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_330.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_330.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_330.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_330.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_330.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_330.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_331.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_331.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_331.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_331.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_331.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_331.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_331.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_331.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_332.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_332.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_332.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_332.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_332.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_332.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_332.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_332.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_333.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_333.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_333.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_333.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_333.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_333.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_333.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_333.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_334.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_334.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_334.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_334.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_334.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_334.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_334.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_334.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_335.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_335.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_335.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_335.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_335.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_335.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_335.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_335.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_336.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_336.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_336.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_336.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_336.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_336.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_336.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_336.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_337.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_337.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_337.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_337.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_337.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_337.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_337.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_337.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_338.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_338.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_338.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_338.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_338.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_338.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_338.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_338.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_339.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_339.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_339.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_339.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_339.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_339.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_339.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_339.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_34.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_34.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_34.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_34.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_34.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_34.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_34.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_34.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_340.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_340.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_340.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_340.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_340.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_340.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_340.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_340.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_341.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_341.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_341.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_341.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_341.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_341.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_341.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_341.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_342.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_342.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_342.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_342.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_342.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_342.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_342.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_342.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_343.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_343.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_343.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_343.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_343.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_343.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_343.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_343.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_344.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_344.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_344.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_344.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_344.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_344.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_344.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_344.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_345.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_345.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_345.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_345.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_345.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_345.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_345.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_345.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_346.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_346.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_346.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_346.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_346.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_346.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_346.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_346.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_347.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_347.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_347.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_347.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_347.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_347.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_347.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_347.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_348.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_348.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_348.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_348.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_348.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_348.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_348.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_348.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_349.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_349.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_349.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_349.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_349.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_349.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_349.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_349.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_35.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_35.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_35.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_35.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_35.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_35.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_35.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_35.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_350.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_350.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_350.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_350.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_350.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_350.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_350.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_350.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_351.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_351.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_351.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_351.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_351.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_351.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_351.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_351.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_352.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_352.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_352.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_352.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_352.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_352.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_352.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_352.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_353.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_353.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_353.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_353.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_353.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_353.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_353.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_353.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_354.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_354.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_354.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_354.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_354.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_354.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_354.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_354.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_355.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_355.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_355.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_355.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_355.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_355.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_355.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_355.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_356.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_356.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_356.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_356.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_356.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_356.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_356.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_356.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_357.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_357.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_357.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_357.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_357.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_357.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_357.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_357.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_358.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_358.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_358.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_358.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_358.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_358.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_358.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_358.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_359.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_359.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_359.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_359.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_359.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_359.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_359.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_359.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_36.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_36.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_36.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_36.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_36.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_36.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_36.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_36.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_360.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_360.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_360.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_360.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_360.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_360.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_360.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_360.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_361.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_361.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_361.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_361.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_361.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_361.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_361.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_361.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_362.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_362.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_362.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_362.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_362.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_362.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_362.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_362.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_363.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_363.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_363.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_363.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_363.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_363.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_363.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_363.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_364.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_364.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_364.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_364.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_364.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_364.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_364.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_364.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_365.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_365.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_365.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_365.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_365.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_365.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_365.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_365.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_366.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_366.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_366.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_366.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_366.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_366.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_366.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_366.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_367.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_367.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_367.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_367.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_367.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_367.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_367.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_367.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_368.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_368.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_368.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_368.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_368.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_368.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_368.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_368.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_369.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_369.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_369.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_369.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_369.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_369.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_369.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_369.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_37.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_37.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_37.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_37.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_37.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_37.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_37.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_37.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_370.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_370.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_370.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_370.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_370.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_370.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_370.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_370.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_371.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_371.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_371.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_371.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_371.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_371.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_371.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_371.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_372.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_372.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_372.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_372.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_372.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_372.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_372.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_372.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_373.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_373.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_373.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_373.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_373.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_373.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_373.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_373.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_374.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_374.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_374.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_374.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_374.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_374.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_374.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_374.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_375.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_375.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_375.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_375.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_375.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_375.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_375.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_375.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_376.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_376.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_376.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_376.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_376.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_376.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_376.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_376.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_377.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_377.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_377.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_377.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_377.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_377.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_377.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_377.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_378.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_378.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_378.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_378.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_378.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_378.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_378.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_378.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_379.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_379.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_379.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_379.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_379.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_379.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_379.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_379.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_38.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_38.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_38.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_38.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_38.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_38.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_38.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_38.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_380.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_380.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_380.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_380.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_380.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_380.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_380.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_380.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_381.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_381.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_381.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_381.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_381.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_381.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_381.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_381.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_382.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_382.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_382.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_382.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_382.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_382.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_382.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_382.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_383.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_383.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_383.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_383.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_383.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_383.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_383.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_383.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_384.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_384.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_384.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_384.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_384.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_384.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_384.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_384.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_385.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_385.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_385.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_385.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_385.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_385.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_385.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_385.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_386.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_386.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_386.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_386.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_386.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_386.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_386.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_386.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_387.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_387.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_387.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_387.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_387.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_387.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_387.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_387.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_388.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_388.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_388.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_388.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_388.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_388.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_388.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_388.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_389.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_389.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_389.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_389.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_389.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_389.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_389.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_389.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_39.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_39.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_39.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_39.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_39.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_39.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_39.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_39.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_390.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_390.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_390.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_390.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_390.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_390.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_390.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_390.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_391.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_391.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_391.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_391.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_391.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_391.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_391.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_391.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_392.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_392.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_392.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_392.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_392.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_392.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_392.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_392.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_393.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_393.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_393.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_393.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_393.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_393.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_393.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_393.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_394.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_394.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_394.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_394.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_394.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_394.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_394.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_394.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_395.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_395.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_395.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_395.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_395.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_395.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_395.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_395.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_396.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_396.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_396.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_396.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_396.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_396.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_396.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_396.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_397.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_397.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_397.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_397.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_397.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_397.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_397.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_397.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_398.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_398.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_398.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_398.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_398.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_398.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_398.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_398.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_399.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_399.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_399.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_399.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_399.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_399.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_399.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_399.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_4.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_4.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_4.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_4.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_4.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_4.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_4.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_4.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_40.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_40.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_40.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_40.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_40.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_40.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_40.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_40.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_400.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_400.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_400.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_400.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_400.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_400.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_400.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_400.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_401.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_401.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_401.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_401.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_401.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_401.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_401.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_401.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_402.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_402.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_402.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_402.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_402.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_402.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_402.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_402.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_403.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_403.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_403.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_403.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_403.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_403.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_403.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_403.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_404.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_404.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_404.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_404.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_404.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_404.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_404.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_404.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_405.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_405.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_405.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_405.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_405.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_405.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_405.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_405.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_406.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_406.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_406.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_406.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_406.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_406.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_406.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_406.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_407.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_407.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_407.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_407.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_407.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_407.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_407.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_407.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_408.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_408.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_408.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_408.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_408.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_408.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_408.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_408.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_409.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_409.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_409.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_409.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_409.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_409.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_409.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_409.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_41.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_41.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_41.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_41.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_41.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_41.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_41.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_41.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_410.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_410.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_410.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_410.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_410.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_410.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_410.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_410.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_411.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_411.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_411.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_411.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_411.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_411.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_411.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_411.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_412.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_412.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_412.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_412.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_412.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_412.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_412.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_412.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_413.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_413.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_413.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_413.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_413.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_413.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_413.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_413.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_414.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_414.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_414.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_414.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_414.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_414.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_414.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_414.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_415.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_415.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_415.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_415.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_415.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_415.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_415.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_415.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_416.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_416.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_416.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_416.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_416.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_416.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_416.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_416.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_417.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_417.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_417.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_417.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_417.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_417.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_417.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_417.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_418.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_418.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_418.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_418.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_418.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_418.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_418.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_418.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_419.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_419.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_419.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_419.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_419.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_419.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_419.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_419.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_42.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_42.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_42.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_42.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_42.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_42.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_42.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_42.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_420.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_420.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_420.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_420.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_420.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_420.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_420.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_420.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_421.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_421.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_421.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_421.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_421.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_421.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_421.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_421.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_422.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_422.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_422.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_422.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_422.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_422.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_422.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_422.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_423.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_423.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_423.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_423.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_423.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_423.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_423.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_423.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_424.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_424.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_424.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_424.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_424.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_424.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_424.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_424.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_425.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_425.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_425.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_425.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_425.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_425.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_425.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_425.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_426.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_426.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_426.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_426.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_426.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_426.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_426.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_426.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_427.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_427.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_427.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_427.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_427.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_427.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_427.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_427.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_428.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_428.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_428.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_428.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_428.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_428.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_428.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_428.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_429.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_429.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_429.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_429.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_429.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_429.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_429.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_429.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_43.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_43.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_43.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_43.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_43.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_43.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_43.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_43.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_430.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_430.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_430.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_430.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_430.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_430.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_430.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_430.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_431.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_431.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_431.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_431.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_431.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_431.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_431.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_431.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_432.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_432.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_432.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_432.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_432.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_432.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_432.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_432.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_433.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_433.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_433.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_433.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_433.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_433.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_433.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_433.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_434.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_434.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_434.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_434.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_434.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_434.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_434.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_434.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_435.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_435.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_435.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_435.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_435.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_435.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_435.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_435.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_436.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_436.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_436.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_436.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_436.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_436.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_436.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_436.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_437.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_437.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_437.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_437.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_437.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_437.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_437.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_437.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_438.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_438.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_438.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_438.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_438.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_438.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_438.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_438.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_439.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_439.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_439.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_439.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_439.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_439.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_439.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_439.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_44.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_44.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_44.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_44.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_44.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_44.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_44.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_44.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_440.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_440.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_440.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_440.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_440.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_440.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_440.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_440.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_441.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_441.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_441.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_441.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_441.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_441.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_441.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_441.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_442.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_442.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_442.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_442.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_442.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_442.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_442.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_442.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_443.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_443.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_443.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_443.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_443.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_443.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_443.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_443.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_444.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_444.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_444.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_444.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_444.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_444.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_444.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_444.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_445.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_445.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_445.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_445.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_445.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_445.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_445.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_445.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_446.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_446.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_446.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_446.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_446.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_446.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_446.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_446.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_447.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_447.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_447.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_447.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_447.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_447.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_447.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_447.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_448.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_448.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_448.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_448.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_448.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_448.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_448.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_448.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_449.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_449.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_449.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_449.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_449.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_449.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_449.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_449.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_45.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_45.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_45.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_45.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_45.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_45.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_45.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_45.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_450.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_450.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_450.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_450.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_450.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_450.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_450.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_450.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_451.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_451.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_451.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_451.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_451.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_451.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_451.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_451.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_452.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_452.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_452.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_452.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_452.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_452.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_452.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_452.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_453.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_453.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_453.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_453.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_453.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_453.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_453.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_453.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_454.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_454.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_454.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_454.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_454.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_454.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_454.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_454.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_455.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_455.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_455.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_455.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_455.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_455.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_455.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_455.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_456.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_456.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_456.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_456.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_456.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_456.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_456.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_456.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_457.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_457.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_457.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_457.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_457.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_457.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_457.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_457.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_458.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_458.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_458.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_458.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_458.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_458.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_458.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_458.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_459.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_459.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_459.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_459.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_459.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_459.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_459.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_459.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_46.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_46.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_46.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_46.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_46.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_46.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_46.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_46.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_460.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_460.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_460.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_460.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_460.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_460.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_460.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_460.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_461.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_461.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_461.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_461.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_461.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_461.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_461.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_461.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_462.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_462.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_462.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_462.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_462.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_462.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_462.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_462.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_463.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_463.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_463.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_463.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_463.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_463.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_463.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_463.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_464.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_464.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_464.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_464.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_464.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_464.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_464.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_464.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_465.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_465.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_465.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_465.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_465.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_465.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_465.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_465.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_466.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_466.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_466.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_466.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_466.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_466.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_466.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_466.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_467.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_467.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_467.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_467.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_467.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_467.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_467.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_467.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_468.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_468.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_468.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_468.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_468.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_468.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_468.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_468.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_469.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_469.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_469.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_469.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_469.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_469.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_469.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_469.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_47.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_47.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_47.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_47.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_47.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_47.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_47.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_47.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_470.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_470.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_470.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_470.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_470.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_470.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_470.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_470.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_471.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_471.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_471.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_471.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_471.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_471.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_471.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_471.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_472.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_472.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_472.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_472.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_472.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_472.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_472.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_472.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_473.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_473.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_473.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_473.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_473.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_473.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_473.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_473.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_474.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_474.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_474.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_474.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_474.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_474.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_474.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_474.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_475.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_475.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_475.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_475.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_475.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_475.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_475.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_475.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_476.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_476.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_476.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_476.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_476.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_476.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_476.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_476.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_477.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_477.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_477.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_477.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_477.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_477.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_477.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_477.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_478.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_478.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_478.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_478.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_478.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_478.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_478.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_478.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_479.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_479.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_479.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_479.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_479.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_479.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_479.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_479.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_48.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_48.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_48.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_48.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_48.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_48.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_48.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_48.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_480.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_480.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_480.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_480.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_480.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_480.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_480.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_480.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_481.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_481.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_481.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_481.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_481.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_481.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_481.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_481.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_482.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_482.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_482.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_482.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_482.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_482.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_482.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_482.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_483.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_483.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_483.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_483.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_483.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_483.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_483.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_483.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_484.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_484.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_484.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_484.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_484.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_484.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_484.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_484.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_485.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_485.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_485.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_485.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_485.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_485.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_485.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_485.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_486.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_486.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_486.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_486.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_486.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_486.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_486.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_486.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_487.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_487.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_487.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_487.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_487.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_487.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_487.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_487.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_488.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_488.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_488.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_488.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_488.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_488.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_488.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_488.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_489.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_489.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_489.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_489.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_489.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_489.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_489.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_489.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_49.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_49.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_49.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_49.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_49.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_49.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_49.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_49.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_490.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_490.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_490.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_490.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_490.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_490.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_490.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_490.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_491.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_491.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_491.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_491.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_491.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_491.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_491.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_491.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_492.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_492.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_492.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_492.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_492.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_492.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_492.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_492.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_493.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_493.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_493.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_493.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_493.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_493.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_493.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_493.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_494.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_494.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_494.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_494.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_494.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_494.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_494.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_494.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_495.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_495.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_495.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_495.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_495.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_495.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_495.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_495.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_496.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_496.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_496.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_496.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_496.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_496.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_496.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_496.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_497.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_497.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_497.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_497.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_497.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_497.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_497.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_497.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_498.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_498.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_498.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_498.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_498.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_498.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_498.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_498.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_499.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_499.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_499.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_499.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_499.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_499.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_499.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_499.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_5.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_5.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_5.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_5.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_5.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_5.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_5.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_5.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_50.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_50.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_50.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_50.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_50.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_50.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_50.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_50.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_500.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_500.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_500.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_500.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_500.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_500.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_500.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_500.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_501.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_501.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_501.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_501.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_501.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_501.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_501.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_501.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_502.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_502.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_502.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_502.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_502.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_502.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_502.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_502.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_503.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_503.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_503.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_503.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_503.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_503.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_503.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_503.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_504.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_504.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_504.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_504.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_504.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_504.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_504.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_504.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_505.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_505.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_505.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_505.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_505.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_505.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_505.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_505.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_506.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_506.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_506.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_506.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_506.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_506.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_506.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_506.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_507.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_507.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_507.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_507.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_507.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_507.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_507.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_507.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_508.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_508.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_508.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_508.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_508.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_508.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_508.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_508.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_509.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_509.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_509.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_509.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_509.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_509.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_509.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_509.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_51.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_51.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_51.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_51.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_51.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_51.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_51.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_51.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_510.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_510.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_510.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_510.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_510.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_510.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_510.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_510.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_511.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_511.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_511.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_511.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_511.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_511.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_511.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_511.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_512.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_512.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_512.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_512.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_512.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_512.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_512.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_512.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_513.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_513.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_513.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_513.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_513.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_513.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_513.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_513.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_514.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_514.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_514.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_514.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_514.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_514.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_514.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_514.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_515.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_515.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_515.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_515.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_515.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_515.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_515.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_515.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_516.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_516.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_516.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_516.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_516.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_516.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_516.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_516.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_517.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_517.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_517.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_517.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_517.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_517.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_517.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_517.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_518.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_518.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_518.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_518.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_518.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_518.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_518.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_518.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_519.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_519.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_519.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_519.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_519.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_519.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_519.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_519.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_52.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_52.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_52.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_52.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_52.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_52.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_52.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_52.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_520.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_520.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_520.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_520.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_520.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_520.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_520.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_520.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_521.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_521.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_521.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_521.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_521.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_521.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_521.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_521.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_522.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_522.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_522.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_522.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_522.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_522.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_522.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_522.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_523.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_523.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_523.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_523.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_523.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_523.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_523.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_523.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_524.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_524.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_524.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_524.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_524.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_524.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_524.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_524.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_525.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_525.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_525.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_525.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_525.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_525.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_525.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_525.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_526.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_526.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_526.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_526.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_526.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_526.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_526.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_526.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_527.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_527.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_527.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_527.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_527.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_527.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_527.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_527.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_528.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_528.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_528.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_528.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_528.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_528.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_528.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_528.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_529.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_529.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_529.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_529.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_529.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_529.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_529.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_529.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_53.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_53.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_53.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_53.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_53.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_53.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_53.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_53.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_530.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_530.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_530.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_530.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_530.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_530.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_530.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_530.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_531.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_531.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_531.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_531.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_531.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_531.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_531.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_531.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_532.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_532.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_532.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_532.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_532.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_532.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_532.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_532.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_533.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_533.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_533.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_533.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_533.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_533.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_533.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_533.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_534.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_534.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_534.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_534.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_534.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_534.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_534.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_534.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_535.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_535.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_535.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_535.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_535.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_535.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_535.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_535.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_536.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_536.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_536.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_536.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_536.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_536.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_536.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_536.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_537.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_537.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_537.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_537.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_537.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_537.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_537.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_537.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_538.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_538.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_538.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_538.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_538.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_538.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_538.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_538.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_539.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_539.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_539.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_539.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_539.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_539.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_539.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_539.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_54.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_54.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_54.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_54.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_54.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_54.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_54.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_54.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_540.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_540.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_540.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_540.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_540.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_540.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_540.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_540.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_541.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_541.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_541.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_541.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_541.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_541.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_541.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_541.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_542.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_542.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_542.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_542.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_542.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_542.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_542.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_542.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_543.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_543.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_543.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_543.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_543.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_543.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_543.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_543.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_544.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_544.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_544.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_544.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_544.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_544.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_544.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_544.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_545.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_545.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_545.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_545.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_545.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_545.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_545.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_545.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_546.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_546.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_546.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_546.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_546.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_546.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_546.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_546.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_547.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_547.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_547.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_547.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_547.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_547.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_547.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_547.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_548.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_548.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_548.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_548.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_548.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_548.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_548.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_548.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_549.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_549.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_549.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_549.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_549.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_549.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_549.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_549.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_55.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_55.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_55.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_55.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_55.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_55.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_55.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_55.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_550.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_550.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_550.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_550.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_550.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_550.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_550.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_550.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_551.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_551.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_551.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_551.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_551.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_551.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_551.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_551.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_552.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_552.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_552.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_552.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_552.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_552.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_552.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_552.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_553.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_553.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_553.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_553.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_553.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_553.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_553.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_553.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_554.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_554.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_554.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_554.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_554.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_554.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_554.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_554.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_555.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_555.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_555.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_555.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_555.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_555.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_555.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_555.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_556.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_556.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_556.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_556.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_556.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_556.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_556.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_556.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_557.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_557.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_557.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_557.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_557.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_557.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_557.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_557.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_558.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_558.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_558.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_558.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_558.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_558.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_558.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_558.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_559.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_559.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_559.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_559.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_559.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_559.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_559.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_559.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_56.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_56.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_56.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_56.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_56.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_56.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_56.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_56.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_560.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_560.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_560.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_560.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_560.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_560.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_560.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_560.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_561.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_561.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_561.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_561.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_561.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_561.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_561.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_561.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_562.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_562.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_562.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_562.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_562.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_562.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_562.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_562.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_563.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_563.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_563.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_563.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_563.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_563.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_563.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_563.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_564.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_564.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_564.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_564.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_564.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_564.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_564.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_564.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_565.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_565.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_565.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_565.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_565.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_565.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_565.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_565.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_566.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_566.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_566.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_566.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_566.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_566.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_566.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_566.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_567.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_567.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_567.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_567.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_567.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_567.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_567.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_567.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_568.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_568.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_568.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_568.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_568.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_568.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_568.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_568.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_569.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_569.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_569.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_569.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_569.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_569.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_569.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_569.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_57.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_57.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_57.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_57.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_57.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_57.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_57.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_57.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_570.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_570.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_570.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_570.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_570.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_570.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_570.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_570.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_571.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_571.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_571.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_571.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_571.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_571.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_571.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_571.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_572.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_572.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_572.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_572.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_572.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_572.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_572.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_572.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_573.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_573.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_573.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_573.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_573.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_573.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_573.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_573.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_574.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_574.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_574.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_574.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_574.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_574.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_574.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_574.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_575.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_575.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_575.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_575.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_575.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_575.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_575.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_575.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_576.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_576.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_576.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_576.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_576.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_576.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_576.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_576.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_577.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_577.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_577.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_577.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_577.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_577.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_577.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_577.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_578.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_578.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_578.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_578.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_578.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_578.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_578.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_578.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_579.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_579.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_579.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_579.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_579.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_579.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_579.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_579.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_58.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_58.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_58.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_58.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_58.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_58.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_58.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_58.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_580.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_580.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_580.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_580.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_580.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_580.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_580.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_580.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_581.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_581.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_581.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_581.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_581.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_581.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_581.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_581.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_582.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_582.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_582.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_582.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_582.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_582.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_582.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_582.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_583.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_583.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_583.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_583.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_583.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_583.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_583.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_583.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_584.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_584.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_584.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_584.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_584.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_584.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_584.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_584.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_585.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_585.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_585.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_585.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_585.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_585.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_585.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_585.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_586.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_586.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_586.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_586.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_586.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_586.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_586.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_586.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_587.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_587.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_587.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_587.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_587.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_587.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_587.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_587.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_588.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_588.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_588.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_588.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_588.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_588.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_588.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_588.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_589.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_589.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_589.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_589.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_589.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_589.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_589.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_589.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_59.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_59.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_59.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_59.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_59.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_59.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_59.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_59.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_590.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_590.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_590.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_590.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_590.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_590.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_590.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_590.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_591.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_591.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_591.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_591.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_591.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_591.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_591.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_591.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_592.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_592.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_592.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_592.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_592.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_592.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_592.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_592.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_593.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_593.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_593.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_593.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_593.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_593.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_593.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_593.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_594.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_594.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_594.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_594.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_594.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_594.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_594.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_594.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_595.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_595.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_595.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_595.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_595.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_595.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_595.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_595.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_596.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_596.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_596.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_596.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_596.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_596.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_596.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_596.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_597.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_597.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_597.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_597.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_597.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_597.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_597.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_597.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_598.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_598.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_598.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_598.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_598.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_598.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_598.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_598.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_599.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_599.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_599.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_599.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_599.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_599.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_599.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_599.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_6.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_6.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_6.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_6.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_6.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_6.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_6.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_6.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_60.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_60.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_60.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_60.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_60.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_60.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_60.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_60.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_600.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_600.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_600.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_600.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_600.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_600.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_600.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_600.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_601.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_601.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_601.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_601.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_601.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_601.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_601.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_601.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_602.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_602.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_602.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_602.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_602.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_602.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_602.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_602.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_603.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_603.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_603.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_603.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_603.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_603.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_603.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_603.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_604.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_604.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_604.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_604.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_604.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_604.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_604.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_604.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_605.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_605.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_605.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_605.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_605.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_605.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_605.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_605.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_606.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_606.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_606.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_606.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_606.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_606.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_606.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_606.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_607.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_607.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_607.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_607.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_607.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_607.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_607.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_607.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_608.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_608.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_608.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_608.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_608.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_608.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_608.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_608.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_609.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_609.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_609.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_609.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_609.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_609.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_609.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_609.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_61.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_61.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_61.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_61.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_61.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_61.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_61.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_61.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_610.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_610.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_610.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_610.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_610.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_610.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_610.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_610.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_611.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_611.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_611.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_611.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_611.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_611.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_611.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_611.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_612.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_612.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_612.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_612.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_612.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_612.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_612.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_612.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_613.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_613.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_613.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_613.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_613.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_613.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_613.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_613.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_614.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_614.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_614.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_614.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_614.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_614.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_614.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_614.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_615.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_615.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_615.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_615.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_615.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_615.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_615.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_615.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_616.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_616.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_616.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_616.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_616.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_616.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_616.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_616.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_617.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_617.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_617.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_617.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_617.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_617.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_617.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_617.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_618.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_618.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_618.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_618.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_618.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_618.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_618.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_618.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_619.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_619.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_619.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_619.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_619.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_619.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_619.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_619.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_62.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_62.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_62.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_62.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_62.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_62.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_62.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_62.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_620.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_620.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_620.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_620.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_620.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_620.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_620.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_620.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_621.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_621.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_621.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_621.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_621.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_621.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_621.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_621.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_622.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_622.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_622.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_622.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_622.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_622.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_622.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_622.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_623.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_623.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_623.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_623.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_623.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_623.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_623.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_623.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_624.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_624.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_624.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_624.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_624.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_624.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_624.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_624.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_625.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_625.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_625.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_625.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_625.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_625.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_625.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_625.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_626.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_626.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_626.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_626.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_626.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_626.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_626.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_626.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_627.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_627.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_627.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_627.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_627.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_627.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_627.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_627.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_628.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_628.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_628.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_628.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_628.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_628.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_628.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_628.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_629.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_629.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_629.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_629.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_629.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_629.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_629.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_629.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_63.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_63.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_63.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_63.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_63.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_63.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_63.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_63.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_630.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_630.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_630.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_630.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_630.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_630.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_630.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_630.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_631.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_631.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_631.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_631.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_631.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_631.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_631.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_631.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_632.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_632.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_632.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_632.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_632.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_632.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_632.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_632.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_633.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_633.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_633.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_633.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_633.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_633.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_633.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_633.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_634.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_634.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_634.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_634.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_634.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_634.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_634.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_634.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_635.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_635.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_635.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_635.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_635.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_635.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_635.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_635.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_636.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_636.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_636.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_636.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_636.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_636.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_636.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_636.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_637.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_637.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_637.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_637.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_637.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_637.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_637.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_637.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_638.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_638.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_638.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_638.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_638.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_638.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_638.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_638.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_639.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_639.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_639.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_639.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_639.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_639.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_639.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_639.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_64.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_64.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_64.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_64.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_64.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_64.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_64.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_64.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_640.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_640.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_640.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_640.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_640.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_640.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_640.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_640.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_641.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_641.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_641.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_641.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_641.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_641.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_641.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_641.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_642.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_642.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_642.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_642.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_642.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_642.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_642.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_642.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_643.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_643.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_643.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_643.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_643.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_643.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_643.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_643.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_644.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_644.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_644.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_644.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_644.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_644.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_644.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_644.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_645.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_645.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_645.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_645.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_645.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_645.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_645.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_645.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_646.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_646.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_646.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_646.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_646.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_646.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_646.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_646.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_647.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_647.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_647.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_647.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_647.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_647.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_647.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_647.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_648.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_648.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_648.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_648.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_648.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_648.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_648.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_648.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_649.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_649.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_649.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_649.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_649.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_649.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_649.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_649.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_65.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_65.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_65.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_65.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_65.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_65.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_65.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_65.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_650.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_650.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_650.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_650.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_650.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_650.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_650.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_650.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_651.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_651.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_651.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_651.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_651.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_651.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_651.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_651.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_652.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_652.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_652.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_652.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_652.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_652.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_652.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_652.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_653.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_653.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_653.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_653.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_653.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_653.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_653.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_653.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_654.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_654.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_654.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_654.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_654.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_654.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_654.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_654.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_655.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_655.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_655.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_655.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_655.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_655.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_655.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_655.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_656.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_656.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_656.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_656.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_656.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_656.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_656.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_656.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_657.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_657.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_657.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_657.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_657.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_657.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_657.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_657.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_658.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_658.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_658.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_658.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_658.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_658.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_658.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_658.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_659.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_659.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_659.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_659.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_659.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_659.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_659.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_659.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_66.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_66.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_66.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_66.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_66.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_66.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_66.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_66.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_660.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_660.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_660.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_660.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_660.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_660.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_660.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_660.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_661.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_661.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_661.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_661.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_661.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_661.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_661.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_661.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_662.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_662.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_662.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_662.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_662.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_662.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_662.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_662.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_663.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_663.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_663.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_663.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_663.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_663.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_663.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_663.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_664.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_664.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_664.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_664.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_664.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_664.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_664.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_664.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_665.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_665.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_665.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_665.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_665.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_665.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_665.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_665.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_666.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_666.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_666.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_666.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_666.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_666.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_666.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_666.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_667.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_667.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_667.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_667.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_667.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_667.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_667.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_667.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_668.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_668.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_668.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_668.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_668.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_668.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_668.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_668.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_669.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_669.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_669.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_669.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_669.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_669.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_669.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_669.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_67.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_67.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_67.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_67.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_67.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_67.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_67.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_67.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_670.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_670.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_670.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_670.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_670.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_670.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_670.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_670.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_671.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_671.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_671.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_671.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_671.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_671.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_671.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_671.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_672.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_672.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_672.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_672.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_672.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_672.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_672.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_672.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_673.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_673.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_673.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_673.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_673.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_673.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_673.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_673.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_674.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_674.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_674.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_674.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_674.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_674.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_674.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_674.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_675.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_675.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_675.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_675.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_675.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_675.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_675.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_675.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_676.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_676.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_676.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_676.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_676.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_676.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_676.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_676.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_677.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_677.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_677.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_677.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_677.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_677.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_677.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_677.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_678.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_678.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_678.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_678.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_678.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_678.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_678.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_678.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_679.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_679.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_679.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_679.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_679.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_679.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_679.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_679.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_68.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_68.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_68.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_68.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_68.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_68.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_68.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_68.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_680.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_680.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_680.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_680.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_680.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_680.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_680.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_680.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_681.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_681.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_681.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_681.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_681.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_681.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_681.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_681.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_682.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_682.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_682.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_682.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_682.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_682.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_682.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_682.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_683.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_683.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_683.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_683.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_683.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_683.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_683.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_683.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_684.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_684.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_684.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_684.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_684.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_684.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_684.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_684.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_685.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_685.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_685.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_685.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_685.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_685.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_685.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_685.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_686.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_686.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_686.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_686.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_686.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_686.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_686.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_686.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_687.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_687.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_687.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_687.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_687.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_687.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_687.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_687.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_688.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_688.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_688.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_688.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_688.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_688.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_688.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_688.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_689.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_689.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_689.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_689.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_689.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_689.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_689.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_689.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_69.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_69.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_69.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_69.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_69.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_69.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_69.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_69.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_7.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_7.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_7.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_7.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_7.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_7.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_7.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_7.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_70.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_70.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_70.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_70.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_70.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_70.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_70.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_70.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_71.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_71.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_71.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_71.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_71.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_71.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_71.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_71.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_72.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_72.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_72.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_72.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_72.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_72.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_72.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_72.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_73.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_73.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_73.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_73.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_73.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_73.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_73.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_73.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_74.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_74.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_74.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_74.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_74.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_74.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_74.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_74.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_75.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_75.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_75.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_75.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_75.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_75.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_75.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_75.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_76.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_76.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_76.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_76.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_76.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_76.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_76.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_76.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_77.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_77.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_77.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_77.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_77.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_77.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_77.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_77.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_78.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_78.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_78.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_78.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_78.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_78.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_78.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_78.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_79.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_79.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_79.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_79.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_79.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_79.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_79.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_79.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_8.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_8.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_8.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_8.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_8.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_8.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_8.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_8.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_80.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_80.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_80.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_80.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_80.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_80.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_80.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_80.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_81.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_81.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_81.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_81.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_81.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_81.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_81.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_81.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_82.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_82.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_82.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_82.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_82.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_82.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_82.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_82.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_83.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_83.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_83.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_83.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_83.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_83.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_83.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_83.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_84.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_84.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_84.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_84.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_84.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_84.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_84.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_84.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_85.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_85.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_85.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_85.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_85.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_85.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_85.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_85.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_86.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_86.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_86.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_86.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_86.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_86.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_86.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_86.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_87.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_87.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_87.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_87.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_87.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_87.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_87.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_87.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_88.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_88.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_88.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_88.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_88.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_88.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_88.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_88.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_89.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_89.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_89.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_89.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_89.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_89.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_89.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_89.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_9.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_9.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_9.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_9.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_9.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_9.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_9.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_9.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_90.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_90.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_90.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_90.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_90.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_90.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_90.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_90.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_91.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_91.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_91.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_91.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_91.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_91.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_91.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_91.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_92.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_92.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_92.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_92.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_92.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_92.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_92.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_92.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_93.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_93.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_93.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_93.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_93.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_93.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_93.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_93.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_94.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_94.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_94.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_94.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_94.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_94.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_94.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_94.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_95.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_95.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_95.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_95.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_95.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_95.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_95.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_95.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_96.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_96.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_96.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_96.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_96.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_96.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_96.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_96.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_97.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_97.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_97.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_97.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_97.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_97.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_97.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_97.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_98.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_98.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_98.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_98.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_98.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_98.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_98.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_98.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_99.tif'], 1), graythresh(imread(['dna' filesep 'img_dna_99.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_99.tif'], 2), graythresh(imread(['dna' filesep 'img_dna_99.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_99.tif'], 3), graythresh(imread(['dna' filesep 'img_dna_99.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['dna' filesep 'img_dna_99.tif'], 4), graythresh(imread(['dna' filesep 'img_dna_99.tif'], 4))))};

cell={@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_1.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_1.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_1.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_1.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_1.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_1.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_1.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_1.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_10.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_10.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_10.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_10.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_10.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_10.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_10.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_10.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_100.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_100.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_100.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_100.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_100.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_100.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_100.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_100.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_101.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_101.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_101.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_101.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_101.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_101.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_101.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_101.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_102.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_102.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_102.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_102.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_102.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_102.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_102.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_102.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_103.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_103.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_103.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_103.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_103.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_103.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_103.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_103.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_104.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_104.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_104.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_104.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_104.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_104.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_104.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_104.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_105.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_105.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_105.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_105.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_105.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_105.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_105.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_105.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_106.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_106.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_106.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_106.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_106.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_106.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_106.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_106.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_107.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_107.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_107.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_107.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_107.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_107.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_107.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_107.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_108.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_108.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_108.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_108.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_108.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_108.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_108.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_108.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_109.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_109.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_109.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_109.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_109.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_109.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_109.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_109.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_11.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_11.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_11.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_11.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_11.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_11.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_11.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_11.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_110.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_110.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_110.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_110.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_110.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_110.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_110.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_110.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_111.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_111.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_111.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_111.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_111.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_111.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_111.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_111.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_112.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_112.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_112.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_112.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_112.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_112.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_112.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_112.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_113.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_113.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_113.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_113.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_113.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_113.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_113.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_113.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_114.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_114.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_114.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_114.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_114.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_114.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_114.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_114.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_115.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_115.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_115.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_115.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_115.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_115.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_115.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_115.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_116.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_116.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_116.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_116.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_116.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_116.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_116.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_116.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_117.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_117.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_117.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_117.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_117.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_117.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_117.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_117.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_118.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_118.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_118.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_118.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_118.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_118.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_118.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_118.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_119.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_119.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_119.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_119.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_119.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_119.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_119.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_119.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_12.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_12.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_12.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_12.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_12.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_12.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_12.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_12.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_120.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_120.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_120.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_120.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_120.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_120.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_120.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_120.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_121.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_121.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_121.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_121.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_121.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_121.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_121.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_121.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_122.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_122.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_122.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_122.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_122.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_122.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_122.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_122.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_123.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_123.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_123.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_123.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_123.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_123.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_123.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_123.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_124.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_124.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_124.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_124.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_124.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_124.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_124.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_124.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_125.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_125.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_125.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_125.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_125.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_125.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_125.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_125.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_126.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_126.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_126.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_126.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_126.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_126.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_126.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_126.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_127.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_127.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_127.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_127.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_127.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_127.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_127.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_127.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_128.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_128.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_128.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_128.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_128.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_128.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_128.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_128.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_129.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_129.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_129.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_129.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_129.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_129.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_129.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_129.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_13.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_13.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_13.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_13.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_13.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_13.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_13.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_13.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_130.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_130.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_130.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_130.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_130.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_130.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_130.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_130.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_131.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_131.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_131.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_131.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_131.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_131.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_131.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_131.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_132.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_132.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_132.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_132.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_132.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_132.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_132.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_132.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_133.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_133.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_133.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_133.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_133.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_133.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_133.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_133.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_134.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_134.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_134.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_134.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_134.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_134.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_134.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_134.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_135.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_135.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_135.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_135.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_135.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_135.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_135.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_135.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_136.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_136.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_136.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_136.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_136.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_136.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_136.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_136.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_137.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_137.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_137.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_137.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_137.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_137.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_137.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_137.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_138.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_138.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_138.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_138.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_138.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_138.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_138.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_138.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_139.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_139.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_139.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_139.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_139.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_139.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_139.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_139.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_14.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_14.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_14.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_14.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_14.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_14.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_14.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_14.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_140.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_140.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_140.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_140.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_140.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_140.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_140.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_140.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_141.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_141.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_141.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_141.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_141.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_141.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_141.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_141.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_142.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_142.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_142.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_142.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_142.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_142.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_142.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_142.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_143.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_143.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_143.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_143.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_143.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_143.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_143.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_143.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_144.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_144.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_144.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_144.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_144.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_144.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_144.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_144.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_145.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_145.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_145.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_145.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_145.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_145.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_145.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_145.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_146.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_146.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_146.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_146.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_146.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_146.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_146.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_146.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_147.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_147.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_147.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_147.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_147.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_147.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_147.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_147.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_148.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_148.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_148.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_148.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_148.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_148.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_148.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_148.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_149.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_149.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_149.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_149.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_149.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_149.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_149.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_149.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_15.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_15.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_15.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_15.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_15.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_15.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_15.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_15.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_150.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_150.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_150.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_150.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_150.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_150.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_150.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_150.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_151.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_151.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_151.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_151.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_151.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_151.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_151.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_151.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_152.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_152.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_152.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_152.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_152.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_152.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_152.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_152.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_153.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_153.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_153.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_153.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_153.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_153.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_153.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_153.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_154.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_154.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_154.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_154.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_154.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_154.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_154.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_154.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_155.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_155.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_155.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_155.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_155.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_155.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_155.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_155.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_156.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_156.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_156.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_156.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_156.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_156.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_156.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_156.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_157.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_157.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_157.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_157.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_157.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_157.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_157.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_157.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_158.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_158.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_158.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_158.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_158.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_158.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_158.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_158.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_159.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_159.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_159.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_159.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_159.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_159.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_159.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_159.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_16.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_16.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_16.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_16.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_16.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_16.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_16.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_16.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_160.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_160.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_160.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_160.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_160.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_160.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_160.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_160.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_161.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_161.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_161.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_161.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_161.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_161.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_161.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_161.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_162.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_162.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_162.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_162.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_162.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_162.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_162.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_162.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_163.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_163.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_163.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_163.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_163.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_163.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_163.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_163.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_164.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_164.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_164.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_164.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_164.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_164.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_164.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_164.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_165.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_165.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_165.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_165.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_165.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_165.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_165.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_165.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_166.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_166.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_166.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_166.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_166.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_166.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_166.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_166.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_167.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_167.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_167.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_167.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_167.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_167.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_167.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_167.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_168.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_168.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_168.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_168.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_168.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_168.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_168.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_168.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_169.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_169.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_169.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_169.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_169.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_169.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_169.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_169.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_17.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_17.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_17.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_17.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_17.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_17.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_17.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_17.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_170.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_170.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_170.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_170.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_170.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_170.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_170.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_170.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_171.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_171.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_171.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_171.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_171.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_171.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_171.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_171.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_172.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_172.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_172.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_172.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_172.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_172.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_172.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_172.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_173.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_173.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_173.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_173.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_173.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_173.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_173.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_173.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_174.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_174.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_174.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_174.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_174.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_174.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_174.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_174.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_175.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_175.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_175.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_175.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_175.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_175.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_175.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_175.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_176.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_176.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_176.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_176.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_176.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_176.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_176.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_176.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_177.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_177.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_177.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_177.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_177.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_177.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_177.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_177.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_178.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_178.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_178.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_178.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_178.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_178.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_178.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_178.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_179.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_179.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_179.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_179.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_179.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_179.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_179.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_179.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_18.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_18.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_18.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_18.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_18.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_18.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_18.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_18.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_180.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_180.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_180.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_180.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_180.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_180.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_180.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_180.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_181.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_181.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_181.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_181.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_181.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_181.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_181.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_181.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_182.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_182.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_182.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_182.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_182.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_182.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_182.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_182.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_183.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_183.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_183.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_183.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_183.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_183.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_183.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_183.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_184.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_184.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_184.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_184.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_184.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_184.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_184.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_184.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_185.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_185.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_185.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_185.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_185.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_185.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_185.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_185.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_186.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_186.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_186.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_186.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_186.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_186.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_186.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_186.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_187.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_187.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_187.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_187.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_187.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_187.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_187.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_187.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_188.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_188.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_188.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_188.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_188.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_188.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_188.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_188.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_189.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_189.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_189.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_189.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_189.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_189.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_189.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_189.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_19.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_19.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_19.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_19.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_19.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_19.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_19.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_19.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_190.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_190.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_190.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_190.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_190.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_190.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_190.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_190.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_191.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_191.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_191.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_191.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_191.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_191.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_191.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_191.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_192.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_192.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_192.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_192.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_192.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_192.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_192.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_192.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_193.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_193.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_193.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_193.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_193.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_193.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_193.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_193.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_194.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_194.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_194.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_194.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_194.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_194.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_194.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_194.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_195.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_195.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_195.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_195.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_195.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_195.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_195.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_195.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_196.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_196.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_196.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_196.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_196.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_196.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_196.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_196.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_197.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_197.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_197.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_197.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_197.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_197.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_197.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_197.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_198.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_198.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_198.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_198.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_198.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_198.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_198.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_198.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_199.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_199.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_199.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_199.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_199.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_199.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_199.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_199.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_2.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_2.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_2.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_2.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_2.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_2.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_2.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_2.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_20.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_20.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_20.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_20.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_20.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_20.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_20.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_20.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_200.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_200.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_200.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_200.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_200.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_200.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_200.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_200.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_201.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_201.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_201.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_201.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_201.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_201.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_201.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_201.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_202.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_202.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_202.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_202.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_202.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_202.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_202.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_202.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_203.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_203.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_203.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_203.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_203.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_203.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_203.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_203.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_204.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_204.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_204.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_204.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_204.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_204.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_204.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_204.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_205.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_205.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_205.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_205.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_205.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_205.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_205.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_205.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_206.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_206.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_206.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_206.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_206.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_206.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_206.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_206.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_207.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_207.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_207.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_207.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_207.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_207.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_207.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_207.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_208.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_208.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_208.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_208.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_208.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_208.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_208.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_208.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_209.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_209.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_209.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_209.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_209.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_209.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_209.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_209.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_21.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_21.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_21.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_21.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_21.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_21.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_21.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_21.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_210.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_210.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_210.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_210.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_210.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_210.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_210.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_210.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_211.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_211.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_211.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_211.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_211.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_211.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_211.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_211.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_212.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_212.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_212.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_212.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_212.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_212.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_212.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_212.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_213.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_213.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_213.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_213.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_213.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_213.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_213.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_213.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_214.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_214.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_214.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_214.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_214.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_214.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_214.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_214.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_215.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_215.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_215.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_215.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_215.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_215.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_215.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_215.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_216.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_216.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_216.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_216.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_216.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_216.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_216.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_216.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_217.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_217.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_217.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_217.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_217.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_217.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_217.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_217.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_218.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_218.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_218.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_218.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_218.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_218.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_218.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_218.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_219.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_219.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_219.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_219.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_219.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_219.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_219.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_219.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_22.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_22.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_22.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_22.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_22.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_22.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_22.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_22.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_220.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_220.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_220.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_220.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_220.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_220.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_220.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_220.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_221.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_221.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_221.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_221.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_221.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_221.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_221.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_221.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_222.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_222.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_222.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_222.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_222.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_222.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_222.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_222.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_223.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_223.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_223.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_223.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_223.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_223.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_223.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_223.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_224.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_224.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_224.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_224.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_224.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_224.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_224.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_224.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_225.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_225.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_225.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_225.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_225.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_225.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_225.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_225.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_226.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_226.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_226.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_226.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_226.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_226.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_226.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_226.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_227.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_227.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_227.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_227.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_227.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_227.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_227.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_227.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_228.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_228.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_228.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_228.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_228.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_228.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_228.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_228.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_229.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_229.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_229.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_229.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_229.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_229.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_229.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_229.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_23.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_23.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_23.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_23.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_23.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_23.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_23.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_23.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_230.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_230.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_230.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_230.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_230.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_230.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_230.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_230.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_231.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_231.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_231.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_231.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_231.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_231.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_231.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_231.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_232.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_232.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_232.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_232.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_232.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_232.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_232.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_232.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_233.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_233.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_233.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_233.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_233.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_233.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_233.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_233.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_234.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_234.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_234.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_234.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_234.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_234.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_234.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_234.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_235.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_235.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_235.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_235.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_235.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_235.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_235.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_235.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_236.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_236.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_236.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_236.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_236.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_236.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_236.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_236.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_237.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_237.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_237.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_237.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_237.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_237.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_237.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_237.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_238.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_238.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_238.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_238.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_238.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_238.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_238.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_238.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_239.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_239.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_239.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_239.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_239.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_239.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_239.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_239.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_24.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_24.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_24.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_24.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_24.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_24.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_24.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_24.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_240.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_240.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_240.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_240.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_240.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_240.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_240.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_240.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_241.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_241.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_241.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_241.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_241.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_241.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_241.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_241.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_242.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_242.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_242.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_242.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_242.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_242.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_242.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_242.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_243.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_243.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_243.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_243.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_243.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_243.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_243.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_243.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_244.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_244.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_244.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_244.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_244.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_244.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_244.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_244.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_245.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_245.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_245.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_245.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_245.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_245.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_245.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_245.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_246.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_246.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_246.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_246.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_246.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_246.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_246.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_246.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_247.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_247.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_247.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_247.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_247.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_247.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_247.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_247.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_248.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_248.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_248.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_248.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_248.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_248.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_248.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_248.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_249.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_249.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_249.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_249.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_249.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_249.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_249.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_249.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_25.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_25.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_25.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_25.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_25.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_25.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_25.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_25.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_250.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_250.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_250.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_250.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_250.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_250.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_250.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_250.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_251.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_251.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_251.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_251.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_251.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_251.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_251.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_251.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_252.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_252.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_252.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_252.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_252.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_252.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_252.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_252.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_253.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_253.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_253.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_253.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_253.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_253.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_253.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_253.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_254.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_254.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_254.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_254.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_254.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_254.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_254.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_254.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_255.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_255.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_255.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_255.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_255.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_255.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_255.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_255.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_256.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_256.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_256.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_256.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_256.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_256.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_256.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_256.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_257.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_257.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_257.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_257.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_257.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_257.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_257.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_257.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_258.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_258.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_258.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_258.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_258.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_258.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_258.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_258.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_259.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_259.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_259.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_259.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_259.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_259.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_259.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_259.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_26.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_26.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_26.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_26.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_26.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_26.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_26.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_26.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_260.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_260.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_260.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_260.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_260.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_260.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_260.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_260.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_261.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_261.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_261.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_261.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_261.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_261.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_261.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_261.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_262.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_262.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_262.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_262.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_262.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_262.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_262.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_262.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_263.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_263.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_263.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_263.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_263.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_263.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_263.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_263.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_264.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_264.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_264.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_264.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_264.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_264.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_264.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_264.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_265.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_265.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_265.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_265.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_265.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_265.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_265.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_265.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_266.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_266.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_266.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_266.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_266.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_266.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_266.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_266.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_267.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_267.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_267.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_267.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_267.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_267.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_267.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_267.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_268.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_268.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_268.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_268.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_268.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_268.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_268.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_268.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_269.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_269.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_269.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_269.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_269.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_269.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_269.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_269.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_27.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_27.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_27.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_27.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_27.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_27.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_27.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_27.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_270.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_270.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_270.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_270.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_270.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_270.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_270.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_270.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_271.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_271.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_271.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_271.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_271.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_271.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_271.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_271.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_272.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_272.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_272.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_272.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_272.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_272.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_272.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_272.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_273.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_273.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_273.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_273.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_273.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_273.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_273.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_273.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_274.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_274.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_274.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_274.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_274.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_274.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_274.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_274.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_275.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_275.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_275.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_275.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_275.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_275.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_275.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_275.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_276.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_276.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_276.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_276.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_276.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_276.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_276.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_276.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_277.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_277.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_277.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_277.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_277.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_277.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_277.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_277.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_278.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_278.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_278.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_278.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_278.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_278.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_278.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_278.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_279.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_279.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_279.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_279.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_279.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_279.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_279.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_279.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_28.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_28.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_28.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_28.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_28.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_28.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_28.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_28.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_280.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_280.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_280.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_280.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_280.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_280.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_280.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_280.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_281.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_281.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_281.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_281.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_281.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_281.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_281.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_281.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_282.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_282.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_282.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_282.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_282.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_282.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_282.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_282.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_283.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_283.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_283.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_283.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_283.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_283.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_283.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_283.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_284.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_284.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_284.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_284.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_284.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_284.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_284.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_284.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_285.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_285.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_285.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_285.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_285.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_285.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_285.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_285.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_286.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_286.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_286.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_286.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_286.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_286.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_286.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_286.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_287.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_287.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_287.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_287.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_287.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_287.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_287.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_287.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_288.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_288.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_288.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_288.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_288.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_288.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_288.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_288.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_289.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_289.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_289.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_289.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_289.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_289.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_289.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_289.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_29.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_29.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_29.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_29.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_29.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_29.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_29.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_29.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_290.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_290.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_290.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_290.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_290.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_290.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_290.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_290.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_291.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_291.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_291.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_291.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_291.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_291.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_291.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_291.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_292.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_292.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_292.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_292.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_292.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_292.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_292.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_292.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_293.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_293.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_293.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_293.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_293.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_293.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_293.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_293.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_294.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_294.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_294.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_294.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_294.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_294.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_294.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_294.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_295.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_295.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_295.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_295.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_295.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_295.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_295.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_295.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_296.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_296.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_296.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_296.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_296.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_296.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_296.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_296.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_297.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_297.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_297.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_297.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_297.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_297.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_297.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_297.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_298.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_298.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_298.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_298.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_298.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_298.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_298.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_298.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_299.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_299.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_299.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_299.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_299.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_299.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_299.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_299.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_3.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_3.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_3.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_3.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_3.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_3.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_3.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_3.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_30.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_30.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_30.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_30.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_30.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_30.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_30.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_30.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_300.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_300.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_300.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_300.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_300.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_300.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_300.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_300.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_301.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_301.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_301.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_301.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_301.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_301.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_301.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_301.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_302.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_302.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_302.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_302.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_302.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_302.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_302.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_302.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_303.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_303.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_303.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_303.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_303.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_303.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_303.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_303.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_304.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_304.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_304.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_304.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_304.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_304.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_304.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_304.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_305.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_305.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_305.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_305.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_305.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_305.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_305.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_305.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_306.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_306.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_306.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_306.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_306.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_306.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_306.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_306.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_307.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_307.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_307.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_307.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_307.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_307.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_307.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_307.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_308.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_308.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_308.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_308.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_308.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_308.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_308.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_308.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_309.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_309.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_309.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_309.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_309.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_309.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_309.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_309.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_31.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_31.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_31.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_31.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_31.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_31.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_31.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_31.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_310.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_310.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_310.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_310.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_310.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_310.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_310.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_310.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_311.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_311.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_311.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_311.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_311.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_311.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_311.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_311.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_312.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_312.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_312.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_312.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_312.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_312.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_312.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_312.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_313.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_313.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_313.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_313.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_313.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_313.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_313.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_313.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_314.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_314.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_314.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_314.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_314.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_314.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_314.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_314.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_315.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_315.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_315.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_315.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_315.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_315.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_315.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_315.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_316.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_316.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_316.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_316.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_316.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_316.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_316.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_316.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_317.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_317.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_317.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_317.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_317.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_317.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_317.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_317.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_318.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_318.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_318.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_318.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_318.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_318.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_318.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_318.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_319.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_319.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_319.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_319.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_319.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_319.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_319.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_319.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_32.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_32.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_32.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_32.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_32.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_32.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_32.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_32.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_320.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_320.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_320.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_320.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_320.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_320.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_320.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_320.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_321.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_321.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_321.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_321.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_321.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_321.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_321.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_321.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_322.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_322.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_322.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_322.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_322.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_322.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_322.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_322.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_323.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_323.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_323.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_323.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_323.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_323.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_323.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_323.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_324.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_324.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_324.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_324.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_324.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_324.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_324.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_324.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_325.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_325.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_325.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_325.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_325.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_325.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_325.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_325.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_326.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_326.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_326.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_326.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_326.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_326.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_326.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_326.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_327.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_327.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_327.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_327.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_327.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_327.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_327.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_327.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_328.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_328.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_328.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_328.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_328.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_328.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_328.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_328.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_329.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_329.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_329.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_329.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_329.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_329.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_329.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_329.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_33.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_33.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_33.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_33.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_33.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_33.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_33.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_33.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_330.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_330.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_330.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_330.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_330.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_330.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_330.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_330.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_331.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_331.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_331.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_331.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_331.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_331.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_331.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_331.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_332.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_332.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_332.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_332.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_332.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_332.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_332.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_332.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_333.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_333.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_333.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_333.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_333.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_333.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_333.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_333.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_334.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_334.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_334.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_334.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_334.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_334.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_334.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_334.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_335.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_335.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_335.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_335.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_335.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_335.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_335.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_335.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_336.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_336.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_336.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_336.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_336.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_336.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_336.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_336.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_337.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_337.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_337.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_337.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_337.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_337.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_337.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_337.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_338.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_338.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_338.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_338.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_338.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_338.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_338.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_338.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_339.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_339.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_339.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_339.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_339.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_339.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_339.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_339.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_34.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_34.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_34.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_34.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_34.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_34.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_34.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_34.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_340.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_340.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_340.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_340.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_340.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_340.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_340.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_340.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_341.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_341.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_341.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_341.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_341.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_341.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_341.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_341.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_342.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_342.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_342.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_342.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_342.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_342.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_342.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_342.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_343.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_343.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_343.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_343.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_343.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_343.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_343.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_343.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_344.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_344.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_344.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_344.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_344.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_344.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_344.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_344.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_345.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_345.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_345.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_345.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_345.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_345.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_345.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_345.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_346.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_346.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_346.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_346.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_346.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_346.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_346.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_346.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_347.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_347.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_347.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_347.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_347.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_347.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_347.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_347.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_348.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_348.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_348.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_348.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_348.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_348.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_348.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_348.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_349.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_349.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_349.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_349.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_349.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_349.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_349.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_349.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_35.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_35.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_35.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_35.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_35.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_35.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_35.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_35.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_350.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_350.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_350.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_350.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_350.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_350.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_350.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_350.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_351.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_351.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_351.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_351.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_351.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_351.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_351.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_351.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_352.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_352.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_352.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_352.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_352.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_352.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_352.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_352.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_353.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_353.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_353.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_353.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_353.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_353.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_353.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_353.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_354.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_354.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_354.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_354.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_354.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_354.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_354.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_354.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_355.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_355.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_355.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_355.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_355.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_355.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_355.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_355.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_356.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_356.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_356.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_356.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_356.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_356.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_356.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_356.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_357.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_357.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_357.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_357.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_357.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_357.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_357.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_357.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_358.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_358.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_358.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_358.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_358.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_358.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_358.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_358.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_359.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_359.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_359.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_359.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_359.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_359.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_359.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_359.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_36.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_36.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_36.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_36.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_36.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_36.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_36.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_36.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_360.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_360.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_360.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_360.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_360.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_360.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_360.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_360.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_361.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_361.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_361.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_361.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_361.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_361.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_361.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_361.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_362.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_362.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_362.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_362.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_362.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_362.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_362.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_362.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_363.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_363.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_363.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_363.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_363.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_363.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_363.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_363.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_364.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_364.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_364.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_364.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_364.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_364.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_364.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_364.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_365.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_365.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_365.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_365.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_365.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_365.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_365.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_365.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_366.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_366.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_366.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_366.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_366.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_366.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_366.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_366.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_367.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_367.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_367.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_367.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_367.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_367.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_367.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_367.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_368.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_368.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_368.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_368.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_368.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_368.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_368.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_368.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_369.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_369.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_369.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_369.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_369.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_369.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_369.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_369.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_37.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_37.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_37.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_37.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_37.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_37.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_37.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_37.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_370.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_370.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_370.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_370.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_370.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_370.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_370.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_370.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_371.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_371.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_371.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_371.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_371.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_371.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_371.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_371.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_372.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_372.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_372.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_372.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_372.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_372.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_372.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_372.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_373.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_373.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_373.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_373.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_373.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_373.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_373.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_373.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_374.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_374.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_374.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_374.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_374.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_374.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_374.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_374.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_375.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_375.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_375.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_375.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_375.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_375.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_375.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_375.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_376.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_376.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_376.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_376.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_376.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_376.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_376.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_376.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_377.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_377.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_377.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_377.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_377.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_377.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_377.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_377.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_378.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_378.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_378.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_378.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_378.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_378.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_378.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_378.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_379.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_379.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_379.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_379.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_379.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_379.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_379.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_379.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_38.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_38.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_38.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_38.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_38.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_38.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_38.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_38.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_380.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_380.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_380.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_380.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_380.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_380.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_380.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_380.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_381.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_381.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_381.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_381.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_381.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_381.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_381.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_381.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_382.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_382.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_382.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_382.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_382.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_382.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_382.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_382.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_383.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_383.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_383.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_383.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_383.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_383.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_383.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_383.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_384.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_384.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_384.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_384.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_384.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_384.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_384.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_384.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_385.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_385.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_385.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_385.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_385.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_385.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_385.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_385.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_386.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_386.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_386.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_386.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_386.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_386.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_386.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_386.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_387.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_387.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_387.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_387.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_387.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_387.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_387.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_387.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_388.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_388.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_388.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_388.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_388.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_388.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_388.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_388.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_389.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_389.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_389.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_389.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_389.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_389.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_389.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_389.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_39.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_39.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_39.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_39.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_39.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_39.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_39.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_39.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_390.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_390.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_390.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_390.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_390.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_390.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_390.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_390.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_391.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_391.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_391.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_391.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_391.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_391.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_391.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_391.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_392.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_392.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_392.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_392.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_392.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_392.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_392.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_392.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_393.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_393.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_393.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_393.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_393.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_393.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_393.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_393.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_394.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_394.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_394.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_394.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_394.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_394.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_394.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_394.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_395.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_395.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_395.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_395.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_395.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_395.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_395.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_395.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_396.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_396.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_396.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_396.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_396.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_396.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_396.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_396.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_397.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_397.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_397.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_397.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_397.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_397.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_397.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_397.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_398.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_398.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_398.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_398.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_398.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_398.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_398.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_398.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_399.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_399.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_399.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_399.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_399.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_399.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_399.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_399.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_4.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_4.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_4.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_4.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_4.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_4.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_4.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_4.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_40.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_40.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_40.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_40.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_40.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_40.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_40.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_40.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_400.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_400.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_400.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_400.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_400.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_400.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_400.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_400.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_401.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_401.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_401.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_401.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_401.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_401.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_401.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_401.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_402.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_402.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_402.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_402.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_402.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_402.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_402.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_402.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_403.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_403.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_403.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_403.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_403.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_403.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_403.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_403.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_404.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_404.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_404.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_404.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_404.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_404.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_404.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_404.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_405.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_405.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_405.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_405.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_405.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_405.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_405.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_405.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_406.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_406.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_406.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_406.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_406.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_406.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_406.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_406.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_407.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_407.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_407.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_407.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_407.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_407.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_407.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_407.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_408.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_408.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_408.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_408.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_408.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_408.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_408.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_408.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_409.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_409.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_409.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_409.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_409.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_409.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_409.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_409.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_41.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_41.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_41.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_41.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_41.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_41.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_41.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_41.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_410.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_410.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_410.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_410.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_410.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_410.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_410.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_410.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_411.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_411.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_411.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_411.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_411.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_411.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_411.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_411.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_412.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_412.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_412.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_412.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_412.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_412.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_412.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_412.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_413.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_413.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_413.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_413.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_413.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_413.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_413.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_413.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_414.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_414.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_414.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_414.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_414.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_414.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_414.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_414.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_415.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_415.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_415.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_415.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_415.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_415.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_415.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_415.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_416.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_416.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_416.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_416.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_416.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_416.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_416.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_416.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_417.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_417.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_417.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_417.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_417.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_417.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_417.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_417.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_418.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_418.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_418.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_418.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_418.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_418.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_418.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_418.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_419.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_419.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_419.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_419.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_419.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_419.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_419.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_419.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_42.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_42.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_42.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_42.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_42.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_42.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_42.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_42.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_420.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_420.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_420.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_420.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_420.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_420.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_420.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_420.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_421.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_421.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_421.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_421.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_421.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_421.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_421.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_421.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_422.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_422.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_422.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_422.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_422.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_422.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_422.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_422.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_423.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_423.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_423.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_423.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_423.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_423.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_423.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_423.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_424.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_424.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_424.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_424.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_424.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_424.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_424.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_424.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_425.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_425.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_425.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_425.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_425.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_425.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_425.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_425.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_426.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_426.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_426.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_426.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_426.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_426.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_426.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_426.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_427.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_427.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_427.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_427.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_427.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_427.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_427.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_427.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_428.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_428.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_428.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_428.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_428.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_428.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_428.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_428.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_429.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_429.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_429.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_429.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_429.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_429.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_429.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_429.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_43.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_43.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_43.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_43.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_43.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_43.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_43.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_43.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_430.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_430.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_430.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_430.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_430.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_430.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_430.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_430.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_431.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_431.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_431.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_431.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_431.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_431.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_431.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_431.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_432.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_432.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_432.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_432.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_432.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_432.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_432.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_432.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_433.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_433.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_433.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_433.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_433.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_433.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_433.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_433.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_434.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_434.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_434.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_434.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_434.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_434.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_434.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_434.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_435.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_435.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_435.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_435.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_435.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_435.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_435.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_435.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_436.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_436.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_436.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_436.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_436.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_436.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_436.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_436.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_437.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_437.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_437.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_437.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_437.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_437.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_437.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_437.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_438.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_438.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_438.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_438.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_438.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_438.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_438.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_438.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_439.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_439.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_439.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_439.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_439.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_439.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_439.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_439.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_44.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_44.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_44.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_44.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_44.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_44.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_44.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_44.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_440.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_440.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_440.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_440.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_440.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_440.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_440.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_440.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_441.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_441.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_441.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_441.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_441.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_441.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_441.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_441.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_442.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_442.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_442.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_442.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_442.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_442.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_442.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_442.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_443.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_443.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_443.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_443.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_443.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_443.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_443.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_443.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_444.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_444.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_444.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_444.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_444.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_444.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_444.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_444.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_445.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_445.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_445.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_445.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_445.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_445.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_445.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_445.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_446.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_446.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_446.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_446.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_446.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_446.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_446.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_446.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_447.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_447.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_447.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_447.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_447.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_447.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_447.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_447.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_448.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_448.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_448.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_448.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_448.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_448.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_448.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_448.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_449.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_449.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_449.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_449.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_449.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_449.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_449.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_449.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_45.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_45.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_45.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_45.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_45.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_45.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_45.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_45.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_450.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_450.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_450.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_450.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_450.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_450.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_450.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_450.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_451.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_451.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_451.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_451.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_451.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_451.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_451.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_451.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_452.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_452.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_452.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_452.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_452.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_452.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_452.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_452.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_453.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_453.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_453.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_453.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_453.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_453.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_453.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_453.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_454.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_454.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_454.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_454.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_454.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_454.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_454.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_454.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_455.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_455.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_455.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_455.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_455.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_455.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_455.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_455.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_456.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_456.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_456.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_456.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_456.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_456.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_456.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_456.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_457.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_457.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_457.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_457.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_457.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_457.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_457.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_457.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_458.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_458.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_458.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_458.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_458.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_458.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_458.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_458.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_459.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_459.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_459.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_459.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_459.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_459.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_459.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_459.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_46.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_46.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_46.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_46.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_46.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_46.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_46.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_46.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_460.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_460.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_460.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_460.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_460.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_460.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_460.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_460.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_461.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_461.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_461.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_461.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_461.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_461.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_461.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_461.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_462.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_462.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_462.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_462.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_462.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_462.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_462.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_462.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_463.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_463.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_463.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_463.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_463.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_463.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_463.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_463.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_464.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_464.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_464.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_464.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_464.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_464.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_464.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_464.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_465.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_465.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_465.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_465.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_465.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_465.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_465.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_465.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_466.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_466.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_466.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_466.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_466.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_466.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_466.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_466.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_467.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_467.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_467.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_467.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_467.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_467.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_467.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_467.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_468.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_468.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_468.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_468.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_468.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_468.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_468.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_468.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_469.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_469.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_469.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_469.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_469.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_469.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_469.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_469.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_47.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_47.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_47.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_47.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_47.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_47.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_47.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_47.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_470.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_470.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_470.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_470.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_470.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_470.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_470.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_470.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_471.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_471.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_471.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_471.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_471.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_471.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_471.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_471.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_472.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_472.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_472.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_472.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_472.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_472.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_472.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_472.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_473.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_473.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_473.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_473.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_473.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_473.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_473.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_473.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_474.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_474.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_474.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_474.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_474.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_474.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_474.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_474.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_475.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_475.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_475.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_475.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_475.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_475.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_475.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_475.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_476.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_476.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_476.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_476.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_476.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_476.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_476.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_476.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_477.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_477.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_477.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_477.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_477.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_477.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_477.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_477.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_478.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_478.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_478.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_478.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_478.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_478.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_478.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_478.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_479.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_479.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_479.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_479.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_479.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_479.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_479.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_479.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_48.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_48.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_48.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_48.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_48.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_48.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_48.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_48.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_480.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_480.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_480.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_480.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_480.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_480.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_480.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_480.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_481.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_481.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_481.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_481.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_481.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_481.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_481.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_481.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_482.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_482.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_482.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_482.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_482.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_482.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_482.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_482.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_483.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_483.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_483.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_483.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_483.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_483.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_483.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_483.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_484.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_484.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_484.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_484.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_484.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_484.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_484.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_484.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_485.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_485.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_485.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_485.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_485.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_485.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_485.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_485.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_486.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_486.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_486.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_486.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_486.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_486.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_486.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_486.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_487.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_487.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_487.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_487.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_487.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_487.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_487.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_487.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_488.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_488.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_488.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_488.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_488.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_488.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_488.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_488.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_489.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_489.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_489.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_489.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_489.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_489.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_489.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_489.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_49.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_49.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_49.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_49.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_49.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_49.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_49.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_49.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_490.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_490.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_490.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_490.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_490.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_490.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_490.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_490.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_491.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_491.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_491.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_491.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_491.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_491.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_491.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_491.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_492.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_492.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_492.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_492.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_492.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_492.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_492.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_492.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_493.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_493.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_493.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_493.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_493.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_493.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_493.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_493.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_494.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_494.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_494.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_494.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_494.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_494.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_494.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_494.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_495.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_495.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_495.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_495.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_495.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_495.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_495.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_495.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_496.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_496.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_496.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_496.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_496.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_496.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_496.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_496.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_497.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_497.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_497.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_497.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_497.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_497.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_497.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_497.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_498.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_498.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_498.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_498.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_498.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_498.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_498.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_498.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_499.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_499.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_499.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_499.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_499.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_499.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_499.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_499.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_5.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_5.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_5.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_5.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_5.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_5.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_5.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_5.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_50.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_50.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_50.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_50.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_50.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_50.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_50.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_50.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_500.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_500.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_500.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_500.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_500.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_500.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_500.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_500.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_501.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_501.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_501.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_501.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_501.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_501.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_501.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_501.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_502.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_502.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_502.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_502.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_502.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_502.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_502.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_502.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_503.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_503.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_503.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_503.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_503.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_503.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_503.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_503.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_504.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_504.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_504.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_504.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_504.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_504.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_504.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_504.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_505.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_505.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_505.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_505.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_505.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_505.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_505.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_505.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_506.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_506.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_506.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_506.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_506.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_506.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_506.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_506.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_507.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_507.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_507.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_507.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_507.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_507.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_507.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_507.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_508.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_508.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_508.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_508.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_508.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_508.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_508.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_508.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_509.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_509.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_509.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_509.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_509.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_509.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_509.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_509.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_51.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_51.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_51.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_51.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_51.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_51.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_51.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_51.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_510.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_510.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_510.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_510.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_510.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_510.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_510.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_510.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_511.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_511.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_511.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_511.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_511.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_511.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_511.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_511.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_512.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_512.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_512.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_512.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_512.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_512.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_512.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_512.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_513.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_513.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_513.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_513.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_513.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_513.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_513.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_513.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_514.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_514.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_514.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_514.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_514.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_514.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_514.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_514.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_515.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_515.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_515.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_515.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_515.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_515.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_515.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_515.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_516.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_516.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_516.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_516.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_516.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_516.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_516.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_516.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_517.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_517.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_517.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_517.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_517.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_517.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_517.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_517.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_518.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_518.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_518.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_518.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_518.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_518.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_518.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_518.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_519.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_519.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_519.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_519.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_519.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_519.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_519.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_519.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_52.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_52.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_52.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_52.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_52.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_52.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_52.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_52.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_520.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_520.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_520.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_520.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_520.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_520.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_520.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_520.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_521.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_521.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_521.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_521.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_521.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_521.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_521.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_521.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_522.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_522.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_522.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_522.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_522.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_522.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_522.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_522.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_523.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_523.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_523.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_523.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_523.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_523.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_523.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_523.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_524.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_524.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_524.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_524.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_524.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_524.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_524.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_524.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_525.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_525.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_525.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_525.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_525.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_525.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_525.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_525.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_526.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_526.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_526.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_526.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_526.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_526.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_526.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_526.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_527.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_527.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_527.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_527.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_527.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_527.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_527.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_527.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_528.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_528.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_528.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_528.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_528.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_528.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_528.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_528.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_529.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_529.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_529.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_529.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_529.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_529.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_529.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_529.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_53.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_53.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_53.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_53.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_53.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_53.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_53.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_53.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_530.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_530.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_530.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_530.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_530.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_530.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_530.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_530.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_531.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_531.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_531.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_531.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_531.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_531.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_531.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_531.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_532.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_532.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_532.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_532.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_532.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_532.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_532.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_532.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_533.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_533.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_533.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_533.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_533.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_533.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_533.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_533.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_534.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_534.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_534.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_534.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_534.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_534.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_534.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_534.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_535.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_535.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_535.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_535.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_535.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_535.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_535.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_535.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_536.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_536.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_536.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_536.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_536.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_536.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_536.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_536.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_537.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_537.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_537.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_537.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_537.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_537.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_537.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_537.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_538.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_538.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_538.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_538.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_538.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_538.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_538.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_538.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_539.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_539.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_539.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_539.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_539.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_539.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_539.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_539.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_54.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_54.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_54.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_54.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_54.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_54.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_54.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_54.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_540.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_540.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_540.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_540.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_540.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_540.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_540.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_540.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_541.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_541.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_541.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_541.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_541.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_541.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_541.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_541.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_542.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_542.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_542.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_542.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_542.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_542.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_542.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_542.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_543.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_543.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_543.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_543.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_543.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_543.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_543.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_543.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_544.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_544.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_544.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_544.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_544.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_544.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_544.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_544.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_545.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_545.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_545.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_545.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_545.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_545.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_545.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_545.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_546.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_546.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_546.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_546.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_546.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_546.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_546.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_546.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_547.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_547.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_547.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_547.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_547.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_547.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_547.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_547.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_548.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_548.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_548.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_548.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_548.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_548.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_548.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_548.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_549.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_549.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_549.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_549.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_549.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_549.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_549.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_549.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_55.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_55.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_55.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_55.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_55.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_55.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_55.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_55.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_550.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_550.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_550.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_550.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_550.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_550.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_550.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_550.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_551.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_551.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_551.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_551.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_551.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_551.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_551.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_551.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_552.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_552.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_552.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_552.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_552.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_552.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_552.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_552.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_553.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_553.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_553.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_553.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_553.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_553.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_553.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_553.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_554.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_554.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_554.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_554.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_554.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_554.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_554.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_554.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_555.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_555.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_555.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_555.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_555.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_555.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_555.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_555.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_556.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_556.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_556.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_556.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_556.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_556.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_556.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_556.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_557.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_557.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_557.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_557.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_557.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_557.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_557.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_557.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_558.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_558.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_558.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_558.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_558.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_558.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_558.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_558.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_559.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_559.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_559.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_559.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_559.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_559.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_559.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_559.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_56.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_56.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_56.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_56.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_56.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_56.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_56.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_56.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_560.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_560.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_560.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_560.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_560.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_560.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_560.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_560.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_561.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_561.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_561.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_561.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_561.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_561.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_561.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_561.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_562.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_562.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_562.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_562.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_562.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_562.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_562.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_562.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_563.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_563.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_563.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_563.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_563.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_563.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_563.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_563.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_564.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_564.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_564.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_564.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_564.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_564.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_564.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_564.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_565.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_565.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_565.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_565.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_565.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_565.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_565.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_565.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_566.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_566.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_566.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_566.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_566.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_566.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_566.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_566.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_567.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_567.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_567.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_567.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_567.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_567.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_567.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_567.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_568.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_568.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_568.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_568.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_568.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_568.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_568.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_568.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_569.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_569.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_569.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_569.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_569.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_569.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_569.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_569.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_57.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_57.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_57.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_57.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_57.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_57.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_57.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_57.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_570.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_570.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_570.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_570.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_570.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_570.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_570.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_570.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_571.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_571.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_571.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_571.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_571.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_571.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_571.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_571.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_572.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_572.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_572.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_572.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_572.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_572.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_572.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_572.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_573.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_573.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_573.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_573.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_573.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_573.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_573.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_573.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_574.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_574.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_574.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_574.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_574.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_574.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_574.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_574.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_575.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_575.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_575.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_575.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_575.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_575.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_575.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_575.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_576.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_576.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_576.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_576.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_576.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_576.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_576.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_576.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_577.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_577.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_577.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_577.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_577.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_577.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_577.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_577.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_578.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_578.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_578.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_578.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_578.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_578.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_578.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_578.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_579.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_579.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_579.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_579.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_579.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_579.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_579.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_579.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_58.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_58.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_58.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_58.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_58.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_58.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_58.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_58.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_580.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_580.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_580.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_580.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_580.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_580.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_580.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_580.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_581.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_581.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_581.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_581.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_581.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_581.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_581.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_581.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_582.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_582.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_582.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_582.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_582.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_582.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_582.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_582.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_583.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_583.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_583.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_583.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_583.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_583.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_583.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_583.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_584.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_584.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_584.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_584.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_584.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_584.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_584.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_584.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_585.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_585.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_585.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_585.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_585.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_585.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_585.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_585.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_586.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_586.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_586.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_586.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_586.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_586.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_586.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_586.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_587.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_587.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_587.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_587.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_587.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_587.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_587.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_587.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_588.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_588.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_588.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_588.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_588.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_588.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_588.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_588.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_589.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_589.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_589.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_589.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_589.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_589.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_589.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_589.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_59.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_59.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_59.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_59.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_59.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_59.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_59.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_59.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_590.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_590.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_590.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_590.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_590.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_590.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_590.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_590.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_591.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_591.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_591.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_591.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_591.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_591.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_591.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_591.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_592.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_592.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_592.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_592.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_592.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_592.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_592.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_592.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_593.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_593.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_593.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_593.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_593.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_593.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_593.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_593.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_594.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_594.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_594.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_594.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_594.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_594.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_594.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_594.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_595.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_595.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_595.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_595.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_595.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_595.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_595.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_595.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_596.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_596.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_596.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_596.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_596.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_596.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_596.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_596.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_597.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_597.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_597.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_597.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_597.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_597.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_597.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_597.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_598.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_598.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_598.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_598.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_598.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_598.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_598.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_598.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_599.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_599.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_599.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_599.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_599.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_599.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_599.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_599.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_6.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_6.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_6.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_6.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_6.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_6.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_6.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_6.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_60.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_60.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_60.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_60.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_60.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_60.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_60.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_60.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_600.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_600.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_600.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_600.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_600.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_600.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_600.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_600.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_601.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_601.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_601.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_601.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_601.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_601.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_601.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_601.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_602.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_602.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_602.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_602.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_602.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_602.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_602.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_602.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_603.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_603.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_603.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_603.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_603.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_603.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_603.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_603.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_604.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_604.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_604.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_604.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_604.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_604.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_604.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_604.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_605.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_605.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_605.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_605.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_605.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_605.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_605.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_605.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_606.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_606.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_606.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_606.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_606.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_606.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_606.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_606.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_607.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_607.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_607.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_607.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_607.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_607.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_607.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_607.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_608.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_608.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_608.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_608.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_608.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_608.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_608.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_608.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_609.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_609.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_609.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_609.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_609.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_609.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_609.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_609.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_61.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_61.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_61.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_61.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_61.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_61.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_61.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_61.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_610.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_610.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_610.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_610.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_610.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_610.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_610.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_610.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_611.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_611.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_611.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_611.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_611.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_611.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_611.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_611.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_612.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_612.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_612.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_612.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_612.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_612.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_612.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_612.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_613.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_613.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_613.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_613.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_613.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_613.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_613.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_613.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_614.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_614.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_614.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_614.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_614.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_614.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_614.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_614.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_615.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_615.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_615.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_615.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_615.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_615.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_615.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_615.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_616.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_616.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_616.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_616.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_616.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_616.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_616.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_616.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_617.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_617.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_617.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_617.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_617.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_617.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_617.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_617.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_618.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_618.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_618.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_618.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_618.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_618.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_618.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_618.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_619.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_619.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_619.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_619.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_619.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_619.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_619.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_619.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_62.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_62.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_62.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_62.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_62.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_62.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_62.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_62.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_620.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_620.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_620.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_620.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_620.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_620.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_620.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_620.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_621.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_621.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_621.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_621.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_621.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_621.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_621.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_621.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_622.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_622.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_622.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_622.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_622.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_622.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_622.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_622.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_623.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_623.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_623.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_623.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_623.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_623.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_623.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_623.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_624.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_624.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_624.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_624.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_624.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_624.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_624.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_624.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_625.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_625.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_625.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_625.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_625.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_625.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_625.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_625.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_626.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_626.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_626.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_626.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_626.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_626.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_626.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_626.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_627.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_627.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_627.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_627.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_627.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_627.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_627.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_627.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_628.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_628.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_628.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_628.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_628.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_628.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_628.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_628.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_629.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_629.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_629.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_629.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_629.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_629.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_629.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_629.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_63.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_63.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_63.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_63.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_63.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_63.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_63.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_63.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_630.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_630.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_630.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_630.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_630.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_630.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_630.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_630.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_631.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_631.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_631.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_631.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_631.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_631.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_631.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_631.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_632.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_632.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_632.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_632.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_632.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_632.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_632.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_632.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_633.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_633.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_633.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_633.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_633.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_633.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_633.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_633.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_634.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_634.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_634.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_634.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_634.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_634.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_634.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_634.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_635.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_635.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_635.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_635.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_635.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_635.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_635.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_635.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_636.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_636.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_636.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_636.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_636.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_636.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_636.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_636.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_637.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_637.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_637.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_637.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_637.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_637.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_637.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_637.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_638.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_638.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_638.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_638.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_638.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_638.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_638.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_638.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_639.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_639.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_639.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_639.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_639.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_639.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_639.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_639.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_64.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_64.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_64.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_64.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_64.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_64.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_64.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_64.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_640.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_640.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_640.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_640.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_640.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_640.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_640.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_640.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_641.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_641.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_641.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_641.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_641.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_641.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_641.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_641.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_642.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_642.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_642.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_642.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_642.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_642.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_642.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_642.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_643.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_643.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_643.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_643.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_643.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_643.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_643.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_643.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_644.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_644.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_644.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_644.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_644.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_644.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_644.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_644.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_645.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_645.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_645.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_645.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_645.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_645.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_645.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_645.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_646.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_646.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_646.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_646.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_646.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_646.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_646.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_646.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_647.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_647.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_647.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_647.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_647.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_647.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_647.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_647.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_648.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_648.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_648.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_648.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_648.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_648.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_648.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_648.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_649.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_649.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_649.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_649.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_649.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_649.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_649.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_649.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_65.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_65.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_65.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_65.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_65.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_65.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_65.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_65.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_650.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_650.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_650.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_650.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_650.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_650.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_650.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_650.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_651.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_651.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_651.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_651.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_651.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_651.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_651.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_651.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_652.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_652.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_652.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_652.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_652.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_652.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_652.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_652.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_653.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_653.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_653.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_653.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_653.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_653.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_653.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_653.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_654.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_654.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_654.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_654.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_654.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_654.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_654.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_654.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_655.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_655.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_655.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_655.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_655.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_655.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_655.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_655.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_656.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_656.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_656.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_656.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_656.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_656.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_656.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_656.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_657.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_657.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_657.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_657.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_657.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_657.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_657.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_657.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_658.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_658.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_658.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_658.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_658.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_658.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_658.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_658.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_659.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_659.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_659.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_659.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_659.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_659.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_659.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_659.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_66.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_66.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_66.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_66.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_66.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_66.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_66.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_66.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_660.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_660.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_660.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_660.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_660.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_660.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_660.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_660.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_661.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_661.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_661.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_661.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_661.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_661.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_661.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_661.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_662.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_662.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_662.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_662.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_662.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_662.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_662.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_662.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_663.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_663.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_663.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_663.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_663.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_663.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_663.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_663.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_664.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_664.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_664.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_664.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_664.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_664.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_664.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_664.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_665.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_665.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_665.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_665.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_665.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_665.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_665.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_665.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_666.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_666.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_666.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_666.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_666.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_666.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_666.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_666.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_667.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_667.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_667.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_667.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_667.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_667.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_667.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_667.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_668.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_668.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_668.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_668.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_668.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_668.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_668.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_668.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_669.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_669.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_669.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_669.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_669.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_669.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_669.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_669.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_67.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_67.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_67.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_67.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_67.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_67.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_67.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_67.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_670.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_670.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_670.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_670.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_670.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_670.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_670.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_670.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_671.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_671.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_671.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_671.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_671.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_671.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_671.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_671.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_672.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_672.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_672.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_672.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_672.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_672.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_672.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_672.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_673.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_673.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_673.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_673.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_673.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_673.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_673.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_673.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_674.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_674.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_674.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_674.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_674.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_674.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_674.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_674.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_675.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_675.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_675.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_675.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_675.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_675.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_675.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_675.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_676.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_676.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_676.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_676.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_676.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_676.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_676.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_676.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_677.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_677.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_677.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_677.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_677.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_677.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_677.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_677.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_678.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_678.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_678.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_678.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_678.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_678.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_678.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_678.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_679.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_679.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_679.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_679.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_679.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_679.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_679.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_679.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_68.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_68.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_68.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_68.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_68.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_68.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_68.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_68.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_680.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_680.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_680.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_680.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_680.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_680.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_680.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_680.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_681.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_681.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_681.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_681.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_681.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_681.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_681.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_681.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_682.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_682.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_682.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_682.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_682.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_682.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_682.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_682.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_683.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_683.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_683.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_683.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_683.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_683.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_683.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_683.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_684.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_684.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_684.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_684.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_684.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_684.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_684.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_684.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_685.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_685.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_685.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_685.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_685.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_685.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_685.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_685.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_686.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_686.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_686.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_686.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_686.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_686.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_686.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_686.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_687.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_687.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_687.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_687.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_687.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_687.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_687.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_687.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_688.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_688.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_688.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_688.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_688.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_688.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_688.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_688.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_689.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_689.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_689.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_689.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_689.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_689.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_689.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_689.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_69.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_69.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_69.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_69.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_69.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_69.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_69.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_69.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_7.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_7.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_7.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_7.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_7.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_7.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_7.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_7.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_70.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_70.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_70.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_70.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_70.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_70.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_70.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_70.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_71.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_71.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_71.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_71.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_71.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_71.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_71.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_71.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_72.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_72.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_72.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_72.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_72.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_72.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_72.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_72.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_73.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_73.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_73.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_73.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_73.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_73.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_73.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_73.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_74.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_74.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_74.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_74.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_74.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_74.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_74.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_74.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_75.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_75.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_75.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_75.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_75.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_75.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_75.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_75.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_76.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_76.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_76.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_76.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_76.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_76.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_76.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_76.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_77.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_77.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_77.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_77.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_77.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_77.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_77.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_77.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_78.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_78.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_78.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_78.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_78.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_78.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_78.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_78.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_79.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_79.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_79.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_79.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_79.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_79.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_79.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_79.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_8.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_8.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_8.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_8.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_8.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_8.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_8.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_8.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_80.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_80.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_80.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_80.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_80.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_80.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_80.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_80.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_81.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_81.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_81.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_81.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_81.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_81.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_81.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_81.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_82.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_82.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_82.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_82.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_82.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_82.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_82.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_82.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_83.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_83.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_83.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_83.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_83.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_83.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_83.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_83.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_84.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_84.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_84.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_84.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_84.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_84.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_84.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_84.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_85.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_85.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_85.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_85.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_85.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_85.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_85.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_85.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_86.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_86.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_86.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_86.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_86.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_86.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_86.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_86.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_87.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_87.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_87.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_87.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_87.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_87.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_87.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_87.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_88.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_88.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_88.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_88.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_88.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_88.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_88.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_88.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_89.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_89.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_89.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_89.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_89.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_89.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_89.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_89.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_9.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_9.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_9.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_9.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_9.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_9.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_9.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_9.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_90.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_90.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_90.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_90.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_90.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_90.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_90.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_90.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_91.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_91.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_91.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_91.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_91.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_91.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_91.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_91.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_92.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_92.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_92.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_92.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_92.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_92.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_92.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_92.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_93.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_93.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_93.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_93.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_93.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_93.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_93.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_93.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_94.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_94.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_94.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_94.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_94.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_94.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_94.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_94.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_95.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_95.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_95.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_95.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_95.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_95.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_95.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_95.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_96.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_96.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_96.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_96.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_96.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_96.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_96.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_96.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_97.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_97.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_97.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_97.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_97.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_97.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_97.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_97.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_98.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_98.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_98.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_98.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_98.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_98.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_98.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_98.tif'], 4)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_99.tif'], 1), graythresh(imread(['cell' filesep 'img_cell_99.tif'], 1)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_99.tif'], 2), graythresh(imread(['cell' filesep 'img_cell_99.tif'], 2)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_99.tif'], 3), graythresh(imread(['cell' filesep 'img_cell_99.tif'], 3)))), ...
@() im2uint8(im2bw(imread(['cell' filesep 'img_cell_99.tif'], 4), graythresh(imread(['cell' filesep 'img_cell_99.tif'], 4))))};

options.model.resolution = [0.049, 0.049];
options.downsampling = [1,1];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

answer = img2slml( dimensionality, dna, cell, [], options );
end%train_EBImages